// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package crontab

import (
	"archive/zip"
	"context"
	"gd_crontab/apis"
	"gd_crontab/errors"
	"gd_crontab/rpc_apis"
	"gd_crontab/rpc_apis/gd_management"
	"fmt"

	"gd_crontab/common.in/config"
	"gd_crontab/common.in/jsonrpc2"
	"gd_crontab/common.in/utils"
	"github.com/astaxie/beego/orm"
	//	_ "github.com/go-sql-driver/mysql"
	"encoding/json"
	"io"
	"io/ioutil"
	"mime"
	"os"
	"path"
	"strconv"
	"strings"
	"time"

	gerrors "errors"

	"github.com/tealeg/xlsx"
	"go.uber.org/zap"
	"gopkg.in/gomail.v2"
)

// db 本地线下统计数据需要将数据插入的数据库
var db orm.Ormer

// TestAuto本地线下统计flag
var TestAuto = false

// timeLayout 时间格式
var timeLayout = "2006-01-02 15:04:05"

var TestExceptionCompute = false
var TestExceptionSql = false
var TestExceptionMail = false

// wait 函数等待到指定小时时间hour，如果isCurrentDay为true，则hour为当天的小时时间，否则hour为明天的小时时间
func wait(hour int, isCurrentDay bool) {
	now := time.Now() // 计算下一个零点
	next := now.Add(time.Hour * 24)
	if isCurrentDay == true {
		next = now
	}
	next = time.Date(next.Year(), next.Month(), next.Day(), hour, 0, 0, 0, next.Location())
	t := time.NewTimer(next.Sub(now))
	<-t.C
}

// setAllCells 设置execel 单元格格式
func setAllCells(file *xlsx.File) {
	style := xlsx.NewStyle()
	font := xlsx.NewFont(10, "Microsoft YaHei")
	style.Font = *font
	style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")

	headerStyle := xlsx.NewStyle()
	fill := xlsx.NewFill("solid", "00B0C4DE", "00000000")
	headerStyle.Font = *font
	headerStyle.Fill = *fill
	headerStyle.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
	sheets := file.Sheets
	for i := 0; i < len(sheets); i++ {
		rows := sheets[i].Rows
		for j := 0; j < len(rows); j++ {
			cells := rows[j].Cells
			for k := 0; k < len(cells); k++ {
				if j < 2 {
					cells[k].SetStyle(headerStyle)
				} else {
					if cells[k].GetStyle().Fill.FgColor == "00FFFF00" {
						continue
					}
					cells[k].SetStyle(style)
				}
			}
		}
	}
}

// isMonday 是否是周一
func isMonday() bool {
	if time.Now().Weekday() == 1 {
		return true
	}
	return false
}

// isFirstDay 是否是月1号
func isFirstDay() bool {
	if _, _, day := time.Now().Date(); day == 1 {
		return true
	}
	return false
}

// getDayExportTimestamp 获取昨日的开始和结束时间戳
func getDayExportTimestamp() (int64, int64) {
	now := time.Now().Format(timeLayout)
	array := strings.Split(now, " ")
	now = fmt.Sprintf("%s 00:00:00", array[0])
	loc, _ := time.LoadLocation("Local")
	end, _ := time.ParseInLocation(timeLayout, now, loc)

	start := end.AddDate(0, 0, -1)

	return start.Unix(), end.Unix()
}

// getWeekExportTimestamp 获取上周的开始和结束时间戳
func getWeekExportTimestamp(weekStart string) (int64, int64) {
	now := time.Now().Format(timeLayout)

	if weekStart != "" {
		loc, _ := time.LoadLocation("Local")
		start, _ := time.ParseInLocation("2006-01-02", weekStart, loc)
		end := start.AddDate(0, 0, 7)
		return start.Unix(), end.Unix()
	}
	if TestAuto {
		t := time.Now()
		for {
			if t.Weekday() == 1 {
				break
			}
			t = t.AddDate(0, 0, -1)
		}
		now = t.Format(timeLayout)
	}
	array := strings.Split(now, " ")
	now = fmt.Sprintf("%s 00:00:00", array[0])
	loc, _ := time.LoadLocation("Local")
	end, _ := time.ParseInLocation(timeLayout, now, loc)
	start := end.AddDate(0, 0, -7)

	return start.Unix(), end.Unix()
}

// getMonthExportTimestamp 获取上月的开始和结束时间戳
func getMonthExportTimestamp(monthStart string) (int64, int64) {
	now := time.Now().Format(timeLayout)
	if monthStart != "" {
		loc, _ := time.LoadLocation("Local")
		start, _ := time.ParseInLocation("2006-01-02", monthStart, loc)
		end := start
		for i := 1; i < 32; i++ {
			end = start.AddDate(0, 0, i)
			if _, _, day := end.Date(); day == 1 {
				break
			}
		}
		return start.Unix(), end.Unix()
	}
	if TestAuto {
		t := time.Now()
		for {
			if _, _, day := t.Date(); day == 1 {
				break
			}
			t = t.AddDate(0, 0, -1)
		}
		now = t.Format(timeLayout)
	}
	array := strings.Split(now, " ")
	now = fmt.Sprintf("%s 00:00:00", array[0])
	loc, _ := time.LoadLocation("Local")
	end, _ := time.ParseInLocation(timeLayout, now, loc)
	start := end.AddDate(0, -1, 0)
	return start.Unix(), end.Unix()
}

// getMerchantIds 获取所有的商户信息，返回商户id-商户名称 map
func getMerchantIds() map[int64]string {
	req := gd_management.ManagementGetMerchantIdsReq{}
	reply, err := rpc_apis.Management.ManagementGetMerchantIds(context.Background(), &req)
	if err != nil {
		l.Error("rpc",
			zap.String("call", "ManagementGetMerchantIds"),
			zap.String("args", utils.MarshalJsonString(req)),
			zap.String("error", err.Error()))
		return nil
	}
	m := map[int64]string{}
	for _, v := range reply.Infos {
		m[v.Id] = v.Name
	}
	return m
}

// makeXlsxHeader 构建月报zip文件内各商户execel文件表头
func makeXlsxHeader(sheet *xlsx.Sheet, timeList string) {
	row := sheet.AddRow()
	dateCell := row.AddCell()
	dateCell.Value = "时间"
	sheet.SetColWidth(0, 0, 20)
	//dateCell.HMerge
	dateCell.VMerge = 1
	cell := row.AddCell()
	cell.Value = "总调用量情况"
	cell.HMerge = 4
	row.AddCell()
	row.AddCell()
	row.AddCell()
	row.AddCell()

	cell = row.AddCell()
	cell.Value = "有效调用量情况"
	cell.HMerge = 10
	row = sheet.AddRow()
	row.AddCell()
	totalCell := row.AddCell()
	totalCell.Value = "调用总量"

	validCell := row.AddCell()
	validCell.Value = "有效量"

	validRateCell := row.AddCell()
	validRateCell.Value = "有效率"

	chargeCell := row.AddCell()
	chargeCell.Value = "计费量"

	chargeRateCell := row.AddCell()
	chargeRateCell.Value = "计费率"

	successCell := row.AddCell()
	successCell.Value = "成功量"

	successRateCell := row.AddCell()
	successRateCell.Value = "成功率"

	queryCell := row.AddCell()
	queryCell.Value = "查得量"

	queryRateCell := row.AddCell()
	queryRateCell.Value = "查得率"

	nonredundantQueryRateCell := row.AddCell()
	nonredundantQueryRateCell.Value = "排重查得率"

	reuseCell := row.AddCell()
	reuseCell.Value = "复用量"

	reuseRateCell := row.AddCell()
	reuseRateCell.Value = "复用率"

	chargeReuseRateCell := row.AddCell()
	chargeReuseRateCell.Value = "计费复用率"

	platformErrorCell := row.AddCell()
	platformErrorCell.Value = "平台异常量"

	platformErrorRateCell := row.AddCell()
	platformErrorRateCell.Value = "平台异常率"

	providerErrorCell := row.AddCell()
	providerErrorCell.Value = "数据源异常量"

	providerErrorRateCell := row.AddCell()
	providerErrorRateCell.Value = "数据源异常率"

	avgElapsedCell := row.AddCell()
	avgElapsedCell.Value = "平均时长"
	tArray := strings.Split(timeList, ",")
	for _, v := range tArray {
		if v == "" {
			continue
		}
		cell := row.AddCell()
		cell.Value = fmt.Sprintf("时长大于%s秒", v)
	}
}

// makeXlsxDayDetailHeader 构建日报明细表头
func makeXlsxDayDetailHeader(sheet *xlsx.Sheet, timeList string) {
	row := sheet.AddRow()
	dateCell := row.AddCell()
	dateCell.Value = "日期"
	sheet.SetColWidth(0, 0, 20)
	dateCell.VMerge = 1

	merchantCell := row.AddCell()
	merchantCell.Value = "商户"
	sheet.SetColWidth(0, 0, 10)
	merchantCell.VMerge = 1
	apiCell := row.AddCell()
	apiCell.Value = "api"
	sheet.SetColWidth(1, 1, 24)
	apiCell.VMerge = 1

	comboTypeCell := row.AddCell()
	comboTypeCell.Value = "套餐类型"
	comboTypeCell.VMerge = 1

	comboTypeCell = row.AddCell()
	comboTypeCell.Value = "套餐调用上限"
	comboTypeCell.VMerge = 1

	comboTypeCell = row.AddCell()
	comboTypeCell.Value = "今日已用量"
	comboTypeCell.VMerge = 1

	comboTypeCell = row.AddCell()
	comboTypeCell.Value = "套餐剩余量"
	comboTypeCell.VMerge = 1

	cell := row.AddCell()
	cell.Value = "总调用量情况"
	cell.HMerge = 4
	row.AddCell()
	row.AddCell()
	row.AddCell()
	row.AddCell()
	cell = row.AddCell()
	cell.Value = "有效调用量情况"
	cell.HMerge = 12
	row = sheet.AddRow()
	row.AddCell()
	row.AddCell()
	row.AddCell()
	row.AddCell()
	row.AddCell()
	row.AddCell()
	row.AddCell()
	totalCell := row.AddCell()
	totalCell.Value = "调用总量"

	validCell := row.AddCell()
	validCell.Value = "有效量"

	validRateCell := row.AddCell()
	validRateCell.Value = "有效率"

	chargeCell := row.AddCell()
	chargeCell.Value = "计费量"

	chargeRateCell := row.AddCell()
	chargeRateCell.Value = "计费率"

	successCell := row.AddCell()
	successCell.Value = "成功量"

	successRateCell := row.AddCell()
	successRateCell.Value = "成功率"

	queryCell := row.AddCell()
	queryCell.Value = "查得量"

	queryRateCell := row.AddCell()
	queryRateCell.Value = "查得率"

	nonredundantQueryRateCell := row.AddCell()
	nonredundantQueryRateCell.Value = "排重查得率"

	reuseCell := row.AddCell()
	reuseCell.Value = "复用量"

	reuseRateCell := row.AddCell()
	reuseRateCell.Value = "复用率"

	chargeReuseRateCell := row.AddCell()
	chargeReuseRateCell.Value = "计费复用率"

	platformErrorCell := row.AddCell()
	platformErrorCell.Value = "平台异常量"

	platformErrorRateCell := row.AddCell()
	platformErrorRateCell.Value = "平台异常率"

	providerErrorCell := row.AddCell()
	providerErrorCell.Value = "数据源异常量"

	providerErrorRateCell := row.AddCell()
	providerErrorRateCell.Value = "数据源异常率"

	avgElapsedCell := row.AddCell()
	avgElapsedCell.Value = "平均时长"

	timeoutCell := row.AddCell()
	timeoutCell.Value = "超时设置"

	timeoutCountCell := row.AddCell()
	timeoutCountCell.Value = "超时数量"

	tArray := strings.Split(timeList, ",")
	for _, v := range tArray {
		if v == "" {
			continue
		}
		cell := row.AddCell()
		cell.Value = fmt.Sprintf("时长大于%s秒", v)
	}
}

// makeXlsxDetailHeader 构建周报和月报明细字段表头
func makeXlsxDetailHeader(sheet *xlsx.Sheet, timeList string, isMonth bool) {
	row := sheet.AddRow()
	merchantCell := row.AddCell()
	if isMonth == false {
		merchantCell.Value = "商户"
		sheet.SetColWidth(0, 0, 10)
	} else {
		merchantCell.Value = "日期"
		sheet.SetColWidth(0, 0, 20)
	}
	merchantCell.VMerge = 1
	apiCell := row.AddCell()
	apiCell.Value = "api"
	sheet.SetColWidth(1, 1, 24)
	apiCell.VMerge = 1
	cell := row.AddCell()
	cell.Value = "总调用量情况"
	if isMonth == false {
		cell.HMerge = 4
		row.AddCell()
		row.AddCell()
		row.AddCell()
		row.AddCell()
	} else {
		cell.HMerge = 2
		row.AddCell()
		row.AddCell()

	}
	cell = row.AddCell()
	cell.Value = "有效调用量情况"
	cell.HMerge = 12
	row = sheet.AddRow()
	row.AddCell()
	row.AddCell()
	totalCell := row.AddCell()
	totalCell.Value = "调用总量"

	validCell := row.AddCell()
	validCell.Value = "有效量"

	validRateCell := row.AddCell()
	validRateCell.Value = "有效率"

	if isMonth == false {
		chargeCell := row.AddCell()
		chargeCell.Value = "计费量"

		chargeRateCell := row.AddCell()
		chargeRateCell.Value = "计费率"
	}

	successCell := row.AddCell()
	successCell.Value = "成功量"

	successRateCell := row.AddCell()
	successRateCell.Value = "成功率"

	queryCell := row.AddCell()
	queryCell.Value = "查得量"

	queryRateCell := row.AddCell()
	queryRateCell.Value = "查得率"

	nonredundantQueryRateCell := row.AddCell()
	nonredundantQueryRateCell.Value = "排重查得率"

	reuseCell := row.AddCell()
	reuseCell.Value = "复用量"

	reuseRateCell := row.AddCell()
	reuseRateCell.Value = "复用率"

	chargeReuseRateCell := row.AddCell()
	chargeReuseRateCell.Value = "计费复用率"

	platformErrorCell := row.AddCell()
	platformErrorCell.Value = "平台异常量"

	platformErrorRateCell := row.AddCell()
	platformErrorRateCell.Value = "平台异常率"

	providerErrorCell := row.AddCell()
	providerErrorCell.Value = "数据源异常量"

	providerErrorRateCell := row.AddCell()
	providerErrorRateCell.Value = "数据源异常率"

	avgElapsedCell := row.AddCell()
	avgElapsedCell.Value = "平均时长"
	tArray := strings.Split(timeList, ",")
	for _, v := range tArray {
		if v == "" {
			continue
		}
		cell := row.AddCell()
		cell.Value = fmt.Sprintf("时长大于%s秒", v)
	}
}

// makeXlsxViewHeader 构建日报，月报，周报总览表头
func makeXlsxViewHeader(sheet *xlsx.Sheet, timeList string, isCharge bool, isDate bool) {
	row := sheet.AddRow()
	if isDate {
		dateCell := row.AddCell()
		dateCell.Value = "日期"
		dateCell.VMerge = 1
	}
	apiCell := row.AddCell()
	apiCell.Value = "api"
	sheet.SetColWidth(0, 0, 24)
	apiCell.VMerge = 1
	cell := row.AddCell()
	cell.Value = "总调用量情况"
	cell.HMerge = 2
	row.AddCell()
	row.AddCell()
	if isCharge {
		cell.HMerge = 4
		row.AddCell()
		row.AddCell()
	}
	cell = row.AddCell()
	cell.Value = "有效调用量情况"
	cell.HMerge = 12
	row = sheet.AddRow()
	row.AddCell()
	if isDate {
		row.AddCell()
	}
	totalCell := row.AddCell()
	totalCell.Value = "调用总量"

	validCell := row.AddCell()
	validCell.Value = "有效量"

	validRateCell := row.AddCell()
	validRateCell.Value = "有效率"

	if isCharge {
		chargeCell := row.AddCell()
		chargeCell.Value = "计费量"

		chargeRateCell := row.AddCell()
		chargeRateCell.Value = "计费率"
	}

	successCell := row.AddCell()
	successCell.Value = "成功量"

	successRateCell := row.AddCell()
	successRateCell.Value = "成功率"

	queryCell := row.AddCell()
	queryCell.Value = "查得量"

	queryRateCell := row.AddCell()
	queryRateCell.Value = "查得率"

	nonredundantQueryRateCell := row.AddCell()
	nonredundantQueryRateCell.Value = "排重查得率"

	reuseCell := row.AddCell()
	reuseCell.Value = "复用量"

	reuseRateCell := row.AddCell()
	reuseRateCell.Value = "复用率"

	chargeReuseRateCell := row.AddCell()
	chargeReuseRateCell.Value = "计费复用率"

	platformErrorCell := row.AddCell()
	platformErrorCell.Value = "平台异常量"

	platformErrorRateCell := row.AddCell()
	platformErrorRateCell.Value = "平台异常率"

	providerErrorCell := row.AddCell()
	providerErrorCell.Value = "数据源异常量"

	providerErrorRateCell := row.AddCell()
	providerErrorRateCell.Value = "数据源异常率"

	avgElapsedCell := row.AddCell()
	avgElapsedCell.Value = "平均时长"
	tArray := strings.Split(timeList, ",")
	for _, v := range tArray {
		if v == "" {
			continue
		}
		cell := row.AddCell()
		cell.Value = fmt.Sprintf("时长大于%s秒", v)
	}
}

// getTimestampFromDate 将日志字符串转为时间戳
func getTimestampFromDate(date string) int64 {
	array := strings.Split(date, " ")
	if len(array) < 1 {
		return 0
	}
	loc, _ := time.LoadLocation("Local")
	t, _ := time.ParseInLocation("2006-01-02", array[0], loc)
	return t.Unix()
}

// sortDate 将统计结果按日期排序
func sortDate(array []apis.LogQueryUserAcessCountExport) []apis.LogQueryUserAcessCountExport {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {

			if getTimestampFromDate(array[i].Date) > getTimestampFromDate(array[j].Date) {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	return array
}

// parseAccessCount 生成月报zip中各商户execel文件
func parseAccessCount(exportInfos []apis.LogQueryUserAcessCountExport, file *xlsx.File, timeList string) error {
	table := make(map[int64][]apis.LogQueryUserAcessCountExport, 0)
	for index, _ := range exportInfos {
		table[exportInfos[index].ApiId] = append(table[exportInfos[index].ApiId], exportInfos[index])

	}
	if len(table) == 0 {
		file.AddSheet("空")
	}
	for _, v := range table {
		k := v[0].ApiName
		sheet, err := file.AddSheet(k + "指标")
		if err != nil {
			return err
		}
		makeXlsxHeader(sheet, timeList)
		sum := apis.LogQueryUserAcessCountExport{}
		sum.Date = "总"
		v = sortDate(v)
		for index, _ := range v {
			row := sheet.AddRow()
			dateCell := row.AddCell()
			dateCell.Value = v[index].Date

			totalCell := row.AddCell()
			sum.Total += v[index].Total
			totalCell.SetInt(int(v[index].Total))

			validCell := row.AddCell()
			sum.Valid += v[index].Valid
			validCell.SetInt(int(v[index].Valid))

			validRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ValidRate, validRateCell)

			chargeCell := row.AddCell()
			sum.Charge += v[index].Charge
			chargeCell.SetInt(int(v[index].Charge))

			style := xlsx.NewStyle()
			font := xlsx.NewFont(10, "Microsoft YaHei")
			style.Font = *font
			style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
			fill := xlsx.NewFill("solid", "00FFFF00", "00000000")
			style.Fill = *fill
			chargeCell.SetStyle(style)

			chargeRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ChargeRate, chargeRateCell)

			successCell := row.AddCell()
			successCell.SetInt(int(v[index].Success))
			sum.Success += v[index].Success

			successRateCell := row.AddCell()
			SetPercentForXlsx(v[index].SuccessRate, successRateCell)

			queryCell := row.AddCell()
			queryCell.SetInt(int(v[index].Query))
			sum.Query += v[index].Query

			queryRateCell := row.AddCell()
			SetPercentForXlsx(v[index].QueryRate, queryRateCell)

			nonRedundantQueryRateCell := row.AddCell()
			SetPercentForXlsx(v[index].NonredundantQueryRate, nonRedundantQueryRateCell)
			sum.NonredundantQuery += v[index].NonredundantQuery
			sum.Nonredundant += v[index].Nonredundant

			reuseCell := row.AddCell()
			reuseCell.SetInt(int(v[index].Reuse))
			sum.Reuse += v[index].Reuse

			style = xlsx.NewStyle()
			font = xlsx.NewFont(10, "Microsoft YaHei")
			style.Font = *font
			style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
			fill = xlsx.NewFill("solid", "00FFFF00", "00000000")
			style.Fill = *fill

			reuseRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ReuseRate, reuseRateCell)
			reuseRateCell.SetStyle(style)

			chargeReuseRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ChargeReuseRate, chargeReuseRateCell)
			sum.ChargeReuse += v[index].ChargeReuse

			platformErrorCell := row.AddCell()
			platformErrorCell.SetInt(int(v[index].PlatformError))
			sum.PlatformError += v[index].PlatformError

			platformErrorRateCell := row.AddCell()
			SetPercentForXlsx(v[index].PlatformErrorRate, platformErrorRateCell)
			providerErrorCell := row.AddCell()
			providerErrorCell.SetInt(int(v[index].ProviderError))
			sum.ProviderError += v[index].ProviderError

			providerErrorRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ProviderErrorRate, providerErrorRateCell)

			avgElapsedCell := row.AddCell()
			elStr := strconv.FormatFloat(v[index].AvgElapsed, 'f', 2, 64)
			el, _ := strconv.ParseFloat(elStr, 64)
			avgElapsedCell.SetFloatWithFormat(el, "0.00")
			sum.SumElapsed += v[index].SumElapsed

			for _, big := range v[index].BigElapsedList {
				cell := row.AddCell()
				cell.SetInt(big)
			}
			sum.BigElapsedList = sumBigElapseds(sum.BigElapsedList, v[index].BigElapsedList)

		}
		parseSum(&sum, sheet)
	}
	table = nil
	return nil
}

// TGdReport 日报数据库结构
type TGdReport struct {
	Id                    int64
	MerchantId            int64
	ApiId                 int64
	MerchantName          string
	Date                  string `json:"date"`
	MerchantDataApiId     int64
	ComboType             int
	Count                 int
	Used                  int
	Remain                int
	ApiName               string `json:"api_name"`
	Total                 int64  `json:"total"`
	Valid                 int64  `json:"valid"`
	Success               int64  `json:"success"`
	Query                 int64  `json:"query"`
	Reuse                 int64  `json:"reuse"`
	ValidRate             string `json:"valid_rate"`
	SuccessRate           string `json:"success_rate"`
	QueryRate             string `json:"query_rate"`
	ReuseRate             string `json:"reuse_rate"`
	AvgElapsed            string `json:"avg_elapsed"`
	PlatformError         int64  `json:"platform_error"`
	PlatformErrorRate     string `json:"platform_error_rate"`
	ProviderError         int64  `json:"provider_error"`
	ProviderErrorRate     string `json:"provider_error_rate"`
	BigElapsedList        string `json:"big_elapsed_list"`
	Charge                int64  `json:"charge"`
	ChargeRate            string `json:"charge_rate"`
	TimeoutInfo           string `json:"timeout_info"`
	TimeoutCount          int    `json:"timeout_count"`
	Repeated              bool   `json:"repeated"`
	NonredundantQuery     int    `json:"nonredundant_query"`
	NonredundantQueryRate string `json:"nonredundant_query_rate"`
	Nonredundant          int    `json:"nonredundant"`
	ChargeReuse           int64  `json:"charge_reuse"`
	ChargeReuseRate       string `json:"charge_reuse_rate"`
}

// addReport 追加日报数据
func addReport(mainfo *MerchantDataApiInfo, reports []TGdReport, merchant string, timeList string, repeated bool) []TGdReport {
	report := TGdReport{}
	data := mainfo.export
	report.Date = data.Date
	report.ApiName = data.ApiName
	report.Total = data.Total
	report.Valid = data.Valid
	report.Success = data.Success
	report.Query = data.Query
	report.Reuse = data.Reuse
	report.ValidRate = data.ValidRate
	report.SuccessRate = data.SuccessRate
	report.QueryRate = data.QueryRate
	report.ReuseRate = data.ReuseRate
	report.AvgElapsed = strconv.FormatFloat(data.AvgElapsed, 'f', 2, 64)
	report.PlatformError = data.PlatformError
	report.PlatformErrorRate = data.PlatformErrorRate
	report.Charge = data.Charge
	report.ChargeRate = data.ChargeRate
	report.MerchantName = merchant
	report.TimeoutInfo = data.TimeoutInfo
	report.TimeoutCount = data.TimeoutCount
	report.ApiId = data.ApiId
	report.MerchantId = data.MerchantId
	report.MerchantDataApiId = mainfo.countInfo.MerchantDataApiId
	report.ComboType = mainfo.countInfo.ComboType
	report.Count = mainfo.countInfo.Count
	report.Remain = mainfo.countInfo.Remain
	report.Used = mainfo.countInfo.Used
	report.ChargeReuse = data.ChargeReuse
	report.Nonredundant = data.Nonredundant
	report.NonredundantQuery = data.NonredundantQuery
	report.ChargeReuseRate = data.ChargeReuseRate
	report.NonredundantQueryRate = data.NonredundantQueryRate
	array := strings.Split(timeList, ",")
	report.Repeated = repeated
	if len(array) == len(data.BigElapsedList) {
		for i, _ := range array {
			if report.BigElapsedList == "" {
				report.BigElapsedList = fmt.Sprintf("%s:%d", array[i], data.BigElapsedList[i])
			} else {
				report.BigElapsedList = fmt.Sprintf("%s;%s:%d", report.BigElapsedList, array[i], data.BigElapsedList[i])
			}
		}
	}

	reports = append(reports, report)
	return reports
}

// MerchantDataApiInfo 数据api信息，export 将要导出到表中的数据指标， countInfo 数据api的套餐和用量信息
type MerchantDataApiInfo struct {
	export    *apis.LogQueryUserAcessCountExport
	countInfo apis.MerchantApiCountInfo
}

// tmpFile 和 tmpSheet用于保存实际日报中不展示的数据
var tmpFile = "/mnt/tmp.xlsx"

var tmpSheet *xlsx.Sheet

// parseAccessCountDayDetailNew 生成日报明细
// exportInfos 日报指标数据
// reports 用于导出到数据库的日报数组
// sheet  execel明细sheet
// timeList 备用（调用时长超过xx秒的记录数）
// merchant 商户名称
// chargeMap api计费信息 api名称-计费数
// date 日报日期
func parseAccessCountDayDetailNew(exportInfos []apis.LogQueryUserAcessCountExport, reports []TGdReport, sheet *xlsx.Sheet, timeList string, merchant string, chargeMap map[string]int64, date string) ([]TGdReport, error) {
	table := make(map[int64][]apis.LogQueryUserAcessCountExport, 0)
	if len(exportInfos) == 0 {
		return reports, nil
	}

	// 生成apiId-指标数据map
	for index, _ := range exportInfos {
		table[exportInfos[index].ApiId] = append(table[exportInfos[index].ApiId], exportInfos[index])

	}

	// 数据api map
	m := map[string][]*MerchantDataApiInfo{}
	for _, v := range table {
		k := v[0].ApiName
		sum := apis.LogQueryUserAcessCountExport{}
		for index, _ := range v {
			for _, countInfo := range v[index].CountInfos {
				// 将基础api 按所属的数据api进行分类，用于后续填写套餐和用量信息（套餐和用量是属于数据api的）
				key := fmt.Sprintf("%d", countInfo.MerchantDataApiId)
				if _, ok := m[key]; ok == false {
					m[key] = []*MerchantDataApiInfo{}
				}
				item := MerchantDataApiInfo{
					export:    &sum,
					countInfo: countInfo,
				}
				m[key] = append(m[key], &item)
			}
			sum.ApiName = v[index].ApiName
			sum.MerchantId = v[index].MerchantId
			sum.ApiId = v[index].ApiId
			sum.TimeoutCount += v[index].TimeoutCount
			sum.TimeoutInfo = v[index].TimeoutInfo
			sum.Total += v[index].Total
			sum.Valid += v[index].Valid
			sum.Charge += v[index].Charge
			sum.Success += v[index].Success
			sum.Query += v[index].Query
			sum.Reuse += v[index].Reuse
			sum.Nonredundant += v[index].Nonredundant
			sum.NonredundantQuery += v[index].NonredundantQuery
			sum.ChargeReuse += v[index].ChargeReuse
			sum.PlatformError += v[index].PlatformError
			sum.ProviderError += v[index].ProviderError
			sum.SumElapsed += v[index].SumElapsed
			sum.BigElapsedList = sumBigElapseds(sum.BigElapsedList, v[index].BigElapsedList)
			sum.NotShow = v[index].NotShow

		}

		// 基础api计费信息累加,用于总览中展示
		if chargeMap != nil {
			if _, ok := chargeMap[k]; ok == false {
				chargeMap[k] = sum.Charge
			} else {
				chargeMap[k] += sum.Charge
			}
		}
	}

	// 计算数据api调用量和余量
	for _, rows := range m {
		used := 0

		// 调用量针对数据api
		for _, mainfo := range rows {
			if mainfo.countInfo.ComboType == 0 {
				continue
			}
			used += int(mainfo.export.Charge)
		}
		for i, mainfo := range rows {
			if mainfo.countInfo.ComboType == 0 {
				continue
			}

			rows[i].countInfo.Used = used
			if rows[i].countInfo.ComboType == 2 {
				rows[i].countInfo.Remain = rows[i].countInfo.Count - used
				if rows[i].countInfo.Remain < 0 {
					rows[i].countInfo.Remain = 0
				}
			}
		}
	}

	// 明细填写
	exist := map[string]int{}
	var row *xlsx.Row
	for _, rows := range m {
		vcols := len(rows)
		for i, mainfo := range rows {
			key := fmt.Sprintf("%d-%d", mainfo.export.MerchantId, mainfo.export.ApiId)
			if _, ok := exist[key]; ok == false {
				exist[key] = 1
			} else {
				exist[key] += 1
			}
			if mainfo.export.NotShow == true && tmpSheet != nil {
				row = tmpSheet.AddRow()
			} else {
				row = sheet.AddRow()
			}
			dateCell := row.AddCell()
			dateCell.Value = date
			merchantCell := row.AddCell()
			merchantCell.Value = merchant
			apiCell := row.AddCell()
			apiCell.Value = mainfo.export.ApiName
			countComboTypeCell := row.AddCell()
			countTotalCell := row.AddCell()
			countUsedCell := row.AddCell()
			countRemainCell := row.AddCell()
			if i == 0 {
				countComboTypeCell.VMerge = vcols - 1
				countTotalCell.VMerge = vcols - 1
				countUsedCell.VMerge = vcols - 1
				countRemainCell.VMerge = vcols - 1

				countInfo := mainfo.countInfo
				if countInfo.ComboType == 0 {
					countComboTypeCell.Value = "免费"
				} else if countInfo.ComboType == 1 {
					countComboTypeCell.Value = "按次"
					countTotalCell.SetInt(countInfo.Count)
					countUsedCell.SetInt(countInfo.Used)
					countRemainCell.SetInt(countInfo.Remain)
				} else {
					countComboTypeCell.Value = "按天"
					countTotalCell.SetInt(countInfo.Count)
					countUsedCell.SetInt(countInfo.Used)
					countRemainCell.SetInt(countInfo.Remain)
				}
			}

			writeCommonTarget(mainfo.export, row, true, true)

			mainfo.export.Date = date

			// 追加日报数据
			if exist[key] > 1 {
				reports = addReport(mainfo, reports, merchant, timeList, true)
			} else {
				reports = addReport(mainfo, reports, merchant, timeList, false)
			}
		}
	}
	table = nil
	return reports, nil
}

// parseAccessCountDetailNew 商户周报明细填写
// exportInfos 周报指标数据
// sheet  execel明细sheet
// timeList 备用（调用时长超过xx秒的记录数）
// merchant 商户名称
// chargeMap api计费信息 api名称-计费数
func parseAccessCountDetailNew(exportInfos []apis.LogQueryUserAcessCountExport, sheet *xlsx.Sheet, timeList string, merchant string, chargeMap map[string]int64) error {
	table := make(map[int64][]apis.LogQueryUserAcessCountExport, 0)

	if len(exportInfos) == 0 {
		return nil
	}
	row := sheet.AddRow()
	merchantCell := row.AddCell()
	merchantCell.Value = merchant

	for index, _ := range exportInfos {
		table[exportInfos[index].ApiId] = append(table[exportInfos[index].ApiId], exportInfos[index])

	}

	//merchantCell.VMerge = len(table) - 1
	count := 0
	for _, v := range table {
		k := v[0].ApiName
		count += 1
		sum := apis.LogQueryUserAcessCountExport{}
		if count == 1 {
			apiCell := row.AddCell()
			apiCell.Value = k
		} else {
			row = sheet.AddRow()
			merchantCell := row.AddCell()
			merchantCell.Value = merchant
			apiCell := row.AddCell()
			apiCell.Value = k
		}
		for index, _ := range v {

			sum.Total += v[index].Total
			sum.Valid += v[index].Valid
			sum.Charge += v[index].Charge
			sum.Success += v[index].Success
			sum.Query += v[index].Query
			sum.Reuse += v[index].Reuse
			sum.Nonredundant += v[index].Nonredundant
			sum.NonredundantQuery += v[index].NonredundantQuery
			sum.ChargeReuse += v[index].ChargeReuse
			sum.PlatformError += v[index].PlatformError
			sum.ProviderError += v[index].ProviderError
			sum.SumElapsed += v[index].SumElapsed
			sum.BigElapsedList = sumBigElapseds(sum.BigElapsedList, v[index].BigElapsedList)

		}
		if chargeMap != nil {
			if _, ok := chargeMap[k]; ok == false {
				chargeMap[k] = sum.Charge
			} else {
				chargeMap[k] += sum.Charge
			}
		}
		writeCommonTarget(&sum, row, true, false)
	}
	table = nil
	return nil
}

// parseAccessCountMonthDetail 生成商户月报明细
func parseAccessCountMonthDetail(reply *apis.LogQueryUserAccessCountExportReply, sheet *xlsx.Sheet, timeList string, start int64) error {
	table := make(map[string][]apis.LogQueryUserAcessCountExport, 0)

	for index, v := range reply.LogQueryUserAcessCount {
		if v.NotShow {
			//continue
		}
		table[reply.LogQueryUserAcessCount[index].Date] = append(table[reply.LogQueryUserAcessCount[index].Date], reply.LogQueryUserAcessCount[index])

	}
	t := time.Unix(start, 0)
	month := t.Month()
	for {
		if t.Month() != month {
			break
		}
		date := t.Format("2006-01-02")
		t = t.AddDate(0, 0, 1)
		row := sheet.AddRow()
		dateCell := row.AddCell()
		dateCell.Value = date
		v, ok := table[date]
		if ok == false {
			cell := row.AddCell()
			cell.Value = "无api调用"
			for i := 0; i < 14; i++ {
				row.AddCell()
			}
			continue
		}
		//dateCell.VMerge = len(v) - 1
		count := 0
		for index, _ := range v {
			count += 1
			if count > 1 {
				row = sheet.AddRow()
				dateCell := row.AddCell()
				dateCell.Value = date
			}

			apiCell := row.AddCell()
			apiCell.Value = v[index].ApiName

			totalCell := row.AddCell()
			totalCell.SetInt(int(v[index].Total))

			validCell := row.AddCell()
			validCell.SetInt(int(v[index].Valid))

			validRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ValidRate, validRateCell)

			successCell := row.AddCell()
			successCell.SetInt(int(v[index].Success))

			successRateCell := row.AddCell()
			SetPercentForXlsx(v[index].SuccessRate, successRateCell)

			queryCell := row.AddCell()
			queryCell.SetInt(int(v[index].Query))

			queryRateCell := row.AddCell()
			SetPercentForXlsx(v[index].QueryRate, queryRateCell)

			nonredundantQueryRateCell := row.AddCell()
			SetPercentForXlsx(v[index].NonredundantQueryRate, nonredundantQueryRateCell)

			reuseCell := row.AddCell()
			reuseCell.SetInt(int(v[index].Reuse))

			reuseRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ReuseRate, reuseRateCell)

			chargeReuseRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ChargeReuseRate, chargeReuseRateCell)

			platformErrorCell := row.AddCell()
			platformErrorCell.SetInt(int(v[index].PlatformError))

			platformErrorRateCell := row.AddCell()
			SetPercentForXlsx(v[index].PlatformErrorRate, platformErrorRateCell)
			providerErrorCell := row.AddCell()
			providerErrorCell.SetInt(int(v[index].ProviderError))

			providerErrorRateCell := row.AddCell()
			SetPercentForXlsx(v[index].ProviderErrorRate, providerErrorRateCell)

			avgElapsedCell := row.AddCell()
			elStr := strconv.FormatFloat(v[index].AvgElapsed, 'f', 2, 64)
			el, _ := strconv.ParseFloat(elStr, 64)
			avgElapsedCell.SetFloatWithFormat(el, "0.00")
			for _, big := range v[index].BigElapsedList {
				cell := row.AddCell()
				cell.SetInt(big)
			}
		}
	}
	table = nil

	return nil
}

// parseAccessCountView 日报，月报，周报总览公用生成逻辑
func parseAccessCountView(reply *apis.LogQueryUserAccessCountExportReply, sheet *xlsx.Sheet, timeList string, chargeMap map[string]int64, date string) error {
	table := make(map[int64][]apis.LogQueryUserAcessCountExport, 0)
	for index, v := range reply.LogQueryUserAcessCount {
		if v.NotShow {
			continue
		}
		table[reply.LogQueryUserAcessCount[index].ApiId] = append(table[reply.LogQueryUserAcessCount[index].ApiId], reply.LogQueryUserAcessCount[index])

	}
	if len(table) == 0 {
		return nil
	}

	for _, v := range table {
		k := v[0].ApiName
		sum := apis.LogQueryUserAcessCountExport{}
		row := sheet.AddRow()
		if date != "" {
			dateCell := row.AddCell()
			dateCell.Value = date
		}
		apiCell := row.AddCell()
		apiCell.Value = k
		for index, _ := range v {
			sum.Total += v[index].Total
			sum.Valid += v[index].Valid
			sum.Success += v[index].Success
			sum.Query += v[index].Query
			sum.Reuse += v[index].Reuse
			sum.PlatformError += v[index].PlatformError
			sum.ProviderError += v[index].ProviderError
			sum.SumElapsed += v[index].SumElapsed
			sum.Nonredundant += v[index].Nonredundant
			sum.NonredundantQuery += v[index].NonredundantQuery
			sum.ChargeReuse += v[index].ChargeReuse
			sum.BigElapsedList = sumBigElapseds(sum.BigElapsedList, v[index].BigElapsedList)
			if chargeMap == nil {
				continue
			}
			if charge, ok := chargeMap[k]; ok {
				sum.Charge = charge
			}
		}
		isCharge := false
		if chargeMap != nil {
			isCharge = true
		}

		writeCommonTarget(&sum, row, isCharge, false)
	}
	table = nil
	return nil
}

// sumBigElapseds 超时记录数累加
func sumBigElapseds(oldList []int, newList []int) []int {
	if len(oldList) == 0 {
		return newList
	}
	if len(oldList) != len(newList) {
		return oldList
	}

	for i, _ := range oldList {
		oldList[i] = oldList[i] + newList[i]
	}

	return oldList
}

// parseSum月报zip中各商户execel文件指标总计
func parseSum(sum *apis.LogQueryUserAcessCountExport, sheet *xlsx.Sheet) {
	row := sheet.AddRow()

	if sum.Total == 0 {
		sum.ValidRate = "0%"
		sum.ChargeRate = "0%"
	} else {
		sum.ValidRate = strconv.FormatFloat(float64(100*sum.Valid)/float64(sum.Total), 'f', 2, 64) + "%"
		sum.ChargeRate = strconv.FormatFloat(float64(100*sum.Charge)/float64(sum.Total), 'f', 2, 64) + "%"
	}

	if sum.Valid == 0 {
		sum.QueryRate = "0%"
		sum.SuccessRate = "0%"
		sum.ReuseRate = "0%"
		sum.PlatformErrorRate = "0%"
		sum.ProviderErrorRate = "0%"
		sum.ChargeReuseRate = "0%"
	} else {
		sum.ChargeReuseRate = strconv.FormatFloat(float64(100*sum.ChargeReuse)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.QueryRate = strconv.FormatFloat(float64(100*sum.Query)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.SuccessRate = strconv.FormatFloat(float64(100*sum.Success)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.ReuseRate = strconv.FormatFloat(float64(100*sum.Reuse)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.PlatformErrorRate = strconv.FormatFloat(float64(100*sum.PlatformError)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.ProviderErrorRate = strconv.FormatFloat(float64(100*sum.ProviderError)/float64(sum.Valid), 'f', 2, 64) + "%"

	}
	if sum.Nonredundant == 0 {
		sum.NonredundantQueryRate = "0%"
	} else {
		sum.NonredundantQueryRate = strconv.FormatFloat(float64(100*sum.NonredundantQuery)/float64(sum.Nonredundant), 'f', 2, 64) + "%"
	}

	dateCell := row.AddCell()
	dateCell.Value = sum.Date

	totalCell := row.AddCell()
	totalCell.SetInt(int(sum.Total))

	validCell := row.AddCell()
	validCell.SetInt(int(sum.Valid))

	validRateCell := row.AddCell()
	SetPercentForXlsx(sum.ValidRate, validRateCell)

	chargeCell := row.AddCell()
	chargeCell.SetInt(int(sum.Charge))

	style := xlsx.NewStyle()
	font := xlsx.NewFont(10, "Microsoft YaHei")
	style.Font = *font
	style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
	fill := xlsx.NewFill("solid", "00FFFF00", "00000000")
	style.Fill = *fill
	chargeCell.SetStyle(style)

	chargeRateCell := row.AddCell()
	SetPercentForXlsx(sum.ChargeRate, chargeRateCell)

	successCell := row.AddCell()
	successCell.SetInt(int(sum.Success))

	successRateCell := row.AddCell()
	SetPercentForXlsx(sum.SuccessRate, successRateCell)

	queryCell := row.AddCell()
	queryCell.SetInt(int(sum.Query))

	queryRateCell := row.AddCell()
	SetPercentForXlsx(sum.QueryRate, queryRateCell)

	nonredundantQueryRateCell := row.AddCell()
	SetPercentForXlsx(sum.NonredundantQueryRate, nonredundantQueryRateCell)

	reuseCell := row.AddCell()
	reuseCell.SetInt(int(sum.Reuse))

	style = xlsx.NewStyle()
	font = xlsx.NewFont(10, "Microsoft YaHei")
	style.Font = *font
	style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
	fill = xlsx.NewFill("solid", "00FFFF00", "00000000")
	style.Fill = *fill

	reuseRateCell := row.AddCell()
	SetPercentForXlsx(sum.ReuseRate, reuseRateCell)
	reuseRateCell.SetStyle(style)

	chargeReuseRateCell := row.AddCell()
	SetPercentForXlsx(sum.ChargeReuseRate, chargeReuseRateCell)
	chargeReuseRateCell.SetStyle(style)

	platformErrorCell := row.AddCell()
	platformErrorCell.SetInt(int(sum.PlatformError))

	platformErrorRateCell := row.AddCell()
	SetPercentForXlsx(sum.PlatformErrorRate, platformErrorRateCell)

	providerErrorCell := row.AddCell()
	providerErrorCell.SetInt(int(sum.ProviderError))

	providerErrorRateCell := row.AddCell()
	SetPercentForXlsx(sum.ProviderErrorRate, providerErrorRateCell)

	avgElapsedCell := row.AddCell()
	sum.AvgElapsed = sum.SumElapsed / float64(sum.Total)
	elStr := strconv.FormatFloat(sum.AvgElapsed, 'f', 2, 64)
	el, _ := strconv.ParseFloat(elStr, 64)
	avgElapsedCell.SetFloatWithFormat(el, "0.00")
	for _, v := range sum.BigElapsedList {
		cell := row.AddCell()
		cell.SetInt(v)
	}
}

// writeCommonTarget 写入日报、月报、周报通用指标
func writeCommonTarget(sum *apis.LogQueryUserAcessCountExport, row *xlsx.Row, charge bool, timeout bool) {

	if sum.Total == 0 {
		sum.ValidRate = "0%"
		sum.ChargeRate = "0%"
	} else {
		sum.ValidRate = strconv.FormatFloat(float64(100*sum.Valid)/float64(sum.Total), 'f', 2, 64) + "%"
		sum.ChargeRate = strconv.FormatFloat(float64(100*sum.Charge)/float64(sum.Total), 'f', 2, 64) + "%"
	}

	if sum.Valid == 0 {
		sum.QueryRate = "0%"
		sum.SuccessRate = "0%"
		sum.ReuseRate = "0%"
		sum.PlatformErrorRate = "0%"
		sum.ProviderErrorRate = "0%"
		sum.ChargeReuseRate = "0%"
	} else {
		sum.QueryRate = strconv.FormatFloat(float64(100*sum.Query)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.ChargeReuseRate = strconv.FormatFloat(float64(100*sum.ChargeReuse)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.SuccessRate = strconv.FormatFloat(float64(100*sum.Success)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.ReuseRate = strconv.FormatFloat(float64(100*sum.Reuse)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.PlatformErrorRate = strconv.FormatFloat(float64(100*sum.PlatformError)/float64(sum.Valid), 'f', 2, 64) + "%"
		sum.ProviderErrorRate = strconv.FormatFloat(float64(100*sum.ProviderError)/float64(sum.Valid), 'f', 2, 64) + "%"
	}
	if sum.Nonredundant == 0 {
		sum.NonredundantQueryRate = "0%"
	} else {
		sum.NonredundantQueryRate = strconv.FormatFloat(float64(100*sum.NonredundantQuery)/float64(sum.Nonredundant), 'f', 2, 64) + "%"
	}

	totalCell := row.AddCell()
	totalCell.SetInt(int(sum.Total))

	validCell := row.AddCell()
	validCell.SetInt(int(sum.Valid))

	validRateCell := row.AddCell()
	SetPercentForXlsx(sum.ValidRate, validRateCell)

	if charge {
		chargeCell := row.AddCell()
		chargeCell.SetInt(int(sum.Charge))

		style := xlsx.NewStyle()
		font := xlsx.NewFont(10, "Microsoft YaHei")
		style.Font = *font
		style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
		fill := xlsx.NewFill("solid", "00FFFF00", "00000000")
		style.Fill = *fill
		chargeCell.SetStyle(style)
	}

	if charge {
		chargeRateCell := row.AddCell()
		SetPercentForXlsx(sum.ChargeRate, chargeRateCell)
	}

	successCell := row.AddCell()
	successCell.SetInt(int(sum.Success))

	successRateCell := row.AddCell()
	SetPercentForXlsx(sum.SuccessRate, successRateCell)

	queryCell := row.AddCell()
	queryCell.SetInt(int(sum.Query))

	queryRateCell := row.AddCell()
	SetPercentForXlsx(sum.QueryRate, queryRateCell)

	nonredundantQueryRateCell := row.AddCell()
	SetPercentForXlsx(sum.NonredundantQueryRate, nonredundantQueryRateCell)

	reuseCell := row.AddCell()
	reuseCell.SetInt(int(sum.Reuse))

	reuseRateCell := row.AddCell()
	SetPercentForXlsx(sum.ReuseRate, reuseRateCell)

	style := xlsx.NewStyle()
	font := xlsx.NewFont(10, "Microsoft YaHei")
	style.Font = *font
	style.Border = *xlsx.NewBorder("thin", "none", "thin", "thin")
	fill := xlsx.NewFill("solid", "00FFFF00", "00000000")
	style.Fill = *fill
	reuseRateCell.SetStyle(style)

	chargeReuseRateCell := row.AddCell()
	SetPercentForXlsx(sum.ChargeReuseRate, chargeReuseRateCell)
	chargeReuseRateCell.SetStyle(style)

	platformErrorCell := row.AddCell()
	platformErrorCell.SetInt(int(sum.PlatformError))

	platformErrorRateCell := row.AddCell()
	SetPercentForXlsx(sum.PlatformErrorRate, platformErrorRateCell)

	providerErrorCell := row.AddCell()
	providerErrorCell.SetInt(int(sum.ProviderError))

	providerErrorRateCell := row.AddCell()
	SetPercentForXlsx(sum.ProviderErrorRate, providerErrorRateCell)

	avgElapsedCell := row.AddCell()
	sum.AvgElapsed = sum.SumElapsed / float64(sum.Total)
	elStr := strconv.FormatFloat(sum.AvgElapsed, 'f', 2, 64)
	el, _ := strconv.ParseFloat(elStr, 64)
	avgElapsedCell.SetFloatWithFormat(el, "0.00")
	if timeout {

		timeoutCell := row.AddCell()
		timeoutCell.Value = fmt.Sprintf("%s", sum.TimeoutInfo)

		timeoutCountCell := row.AddCell()
		timeoutCountCell.SetInt(sum.TimeoutCount)
	}
	for _, v := range sum.BigElapsedList {
		cell := row.AddCell()
		cell.SetInt(v)
	}
}

// sendMailExec 发送报表邮件执行函数
func sendMailExec(user, password, host, to, subject string, files []string) error {
	hostIp := ""
	hostPort := ""
	if len(files) == 0 {
		return nil
	}
	array := strings.Split(host, ":")
	if len(array) > 1 {
		hostIp = array[0]
		hostPort = array[1]
	} else {
		hostIp = host
		hostPort = "465"
	}

	mailConn := map[string]string{
		"user": user,
		"pass": password,
		"host": hostIp,
		"port": hostPort,
	}

	port, _ := strconv.Atoi(mailConn["port"])
	mailTo := strings.Split(to, ";")
	m := gomail.NewMessage()
	m.SetHeader("From", "getensh"+"<"+mailConn["user"]+">")
	m.SetHeader("To", mailTo...)
	m.SetHeader("Subject", subject)
	for _, v := range files {
		if v != "" {
			newname := path.Base(v)
			m.Attach(v, gomail.Rename(newname),
				gomail.SetHeader(map[string][]string{
					"Content-Disposition": []string{
						fmt.Sprintf(`attachment; filename="%s"`, mime.QEncoding.Encode("UTF-8", newname)),
					}}))
		}
	}

	d := gomail.NewDialer(mailConn["host"], port, mailConn["user"], mailConn["pass"])

	err := d.DialAndSend(m)
	return err
}

// sendExceptionMail 发送异常邮件
func sendExceptionMail(to, host, user, password, msg string) error {
	hostIp := ""
	hostPort := ""
	array := strings.Split(host, ":")
	if len(array) > 1 {
		hostIp = array[0]
		hostPort = array[1]
	} else {
		hostIp = host
		hostPort = "465"
	}

	mailConn := map[string]string{
		"user": user,
		"pass": password,
		"host": hostIp,
		"port": hostPort,
	}

	port, _ := strconv.Atoi(mailConn["port"])
	mailTo := strings.Split(to, ";")
	m := gomail.NewMessage()
	m.SetHeader("From", "getensh"+"<"+mailConn["user"]+">")
	m.SetHeader("To", mailTo...)
	m.SetHeader("Subject", "日报异常")
	m.SetBody("text/html", msg)
	d := gomail.NewDialer(mailConn["host"], port, mailConn["user"], mailConn["pass"])

	err := d.DialAndSend(m)
	return err

}

func sendMail(to, host, user, password string, files ...string) error {
	err := sendMailExec(user,
		password,
		host,
		to, "gd日志报表", files)
	if err != nil {
		l.Error("func",
			zap.String("call", "sendMailExec"),
			zap.String("args",
				fmt.Sprintf("password:%s,host:%s,to:%s,subject:%s,files:%v\n",
					password, host, to, "gd日志报表", files)),
			zap.String("error", err.Error()))
	}
	return err
}

func makeZip(zipFile string, fileList []string) string {
	// 创建 zip 包文件
	var retErr error

	defer func() {
		if retErr != nil {
			l.Error("func",
				zap.String("call", "makeZip"),
				zap.String("error", retErr.Error()))
		}
	}()

	fw, err := os.Create(zipFile)
	if err != nil {
		retErr = err
		return ""
	}
	defer fw.Close()

	// 实例化新的 zip.Writer
	zw := zip.NewWriter(fw)
	defer zw.Close()

	for _, fileName := range fileList {
		fr, err := os.Open(fileName)
		if err != nil {
			retErr = err
			return ""
		}
		fi, err := fr.Stat()
		if err != nil {
			retErr = err
			return ""
		}
		// 写入文件的头信息
		fh, err := zip.FileInfoHeader(fi)
		w, err := zw.CreateHeader(fh)
		if err != nil {
			retErr = err
			return ""
		}
		// 写入文件内容
		_, err = io.Copy(w, fr)
		if err != nil {
			retErr = err
			return ""
		}
	}
	return zipFile
}

func remove(files ...string) {
	for _, file := range files {
		if file == "" {
			continue
		}
		os.Remove(file)
	}
}

func insertReports(reports []TGdReport, date string, apiId int64) error {
	if len(reports) == 0 {
		return nil
	}
	if apiId > 0 {
		oldTabs := []TGdReport{}
		db.Raw("select * from t_gd_report where date=?", date).QueryRows(&oldTabs)

		m := map[string]*TGdReport{}
		for i, v := range oldTabs {
			key := fmt.Sprintf("%d-%d-%d", v.MerchantId, v.ApiId, v.MerchantDataApiId)
			m[key] = &oldTabs[i]
		}
		for i, v := range reports {
			key := fmt.Sprintf("%d-%d-%d", v.MerchantId, v.ApiId, v.MerchantDataApiId)
			if _, ok := m[key]; ok {
				reports[i].ComboType = m[key].ComboType
				reports[i].Count = m[key].Count
				reports[i].Used = m[key].Used
				reports[i].Remain = m[key].Remain
			}

			db.Raw("delete from t_gd_report where date=? and merchant_id=? and merchant_data_api_id=? and api_id=?", date, v.MerchantId, v.MerchantDataApiId, v.ApiId).Exec()
			_, err := db.Insert(&reports[i])
			if err != nil {
				return err
			}
		}
		return nil
	}
	db.Raw("delete from t_gd_report where date=?", date).Exec()
	_, err := db.InsertMulti(len(reports), &reports)
	if err != nil {
		return err
	}
	return nil
}

// SetPercentForXlsx 单元格设置百分率
func SetPercentForXlsx(percent string, cell *xlsx.Cell) {
	s := strings.TrimRight(percent, "%")

	// 对零单独处理
	f, _ := strconv.ParseFloat(s, 64)
	if f == 0 {
		cell.SetFloatWithFormat(0.0000, "0.00%")
		return
	}

	// 将百分比转为小数形式
	f = f / 100.0000
	fstring := strconv.FormatFloat(f, 'f', 4, 64)
	ffloat, _ := strconv.ParseFloat(fstring, 64)

	// 设置单元格
	cell.SetFloatWithFormat(ffloat, "0.00%")
}

func exportDay(start, end, merchantId, apiId int64, isInsert bool) (string, error) {
	if TestExceptionCompute {
		return "", gerrors.New("异常邮件测试:商户日报统计失败")
	}
	if TestExceptionSql {
		return "", gerrors.New("异常邮件测试：sql执行失败")
	}
	file := xlsx.NewFile()
	date := time.Unix(start, 0).Format("2006-01-02")
	detailSheet, err := file.AddSheet("明细")
	if err != nil {
		l.Error("func",
			zap.String("call", "file.AddSheet"),
			zap.String("args", "明细"),
			zap.String("error", err.Error()))
		return "", err
	}
	makeXlsxDayDetailHeader(detailSheet, "")
	chargeMap := map[string]int64{}
	reports := []TGdReport{}

	merchantMap := map[int64][]apis.LogQueryUserAcessCountExport{}
	lastMerchantMap := map[int64][]apis.LogQueryUserAcessCountExport{}
	// 所有商户报表
	sheet, err := file.AddSheet("总览")
	if err != nil {
		l.Error("func",
			zap.String("call", "file.AddSheet"),
			zap.String("args", "总览"),
			zap.String("error", err.Error()))
		return "", err
	}
	makeXlsxViewHeader(sheet, "", true, true)
	mreq := apis.LogQueryUserAccessCountExportReq{}
	reply := apis.LogQueryUserAccessCountExportReply{}

	mreq.StartTimestamp = start
	mreq.EndTimestamp = end
	mreq.MerchantId = merchantId
	mreq.NeedCount = true
	mreq.GroupByMerchant = true
	if apiId != 0 {
		mreq.ApiIdList = append(mreq.ApiIdList, apiId)
	}
	if time.Now().Unix()-start < 3600*26 {
		mreq.TabName = "t_gd_access_log_day"
	}
	err = LogQueryUserAccessCountExport(context.Background(), &mreq, &reply)
	if err != nil {
		l.Error("func",
			zap.String("call", "LogQueryUserAccessCountExport"),
			zap.String("args", utils.MarshalJsonString(mreq)),
			zap.String("error", err.Error()))
		return "", err
	}
	for _, v := range reply.LogQueryUserAcessCount {
		if v.NotShow == true {
			//continue
		}
		merchantMap[v.MerchantId] = append(merchantMap[v.MerchantId], v)
	}
	for id, array := range merchantMap {
		name := array[0].MerchantName
		if strings.Contains(name, "成都小时代科技有限公司") || name == "hj-test" || name == "DB" || name == "GD" || name == "DC" {
			lastMerchantMap[id] = array
			continue
		}
		if reports, err = parseAccessCountDayDetailNew(array, reports, detailSheet, "", name, chargeMap, date); err != nil {
			return "", err
		}
	}
	for _, array := range lastMerchantMap {
		name := array[0].MerchantName
		if reports, err = parseAccessCountDayDetailNew(array, reports, detailSheet, "", name, chargeMap, date); err != nil {
			return "", err
		}
	}
	if isInsert {
		err = insertReports(reports, date, apiId)
		if err != nil {
			return "", err
		}
	}

	if err := parseAccessCountView(&reply, sheet, mreq.TimeList, chargeMap, date); err != nil {
		return "", err
	}
	filename := fmt.Sprintf("gd平台日报-%s", time.Unix(start, 0).Format("2006-01-02"))
	path := config.Conf.ReportLocalDir + filename + ".xlsx"
	setAllCells(file)
	err = file.Save(path)
	if err != nil {
		l.Error("func",
			zap.String("call", "save excel"),
			zap.String("file_name", path),
			zap.String("error", err.Error()))
	}
	return path, err
}

func getUserAccessCountExportFromReportGroupByApi(startDate, endDate string) (*apis.LogQueryUserAccessCountExportReply, error) {
	tabs := []TGdReport{}
	ret := &apis.LogQueryUserAccessCountExportReply{}
	o := orm.NewOrm()
	sql := fmt.Sprintf("select * from t_gd_report where date >= '%s' and date <='%s' and repeated=0", startDate, endDate)
	_, err := o.Raw(sql).QueryRows(&tabs)

	if err != nil {
		l.Error("sql",
			zap.String("call", "getUserAccessCountExportFromReport"),
			zap.String("args", sql),
			zap.String("error", err.Error()))
		return nil, errors.DataBaseError
	}
	dateApiMap := map[string][]TGdReport{}
	for _, v := range tabs {
		key := fmt.Sprintf("%s-%d", v.Date, v.ApiId)
		dateApiMap[key] = append(dateApiMap[key], v)
	}
	for _, v := range dateApiMap {
		item := apis.LogQueryUserAcessCountExport{}
		for _, sub := range v {
			item.Date = sub.Date
			item.Total += sub.Total
			item.ApiId = sub.ApiId
			item.Success += sub.Success
			avg, _ := strconv.ParseFloat(sub.AvgElapsed, 64)
			item.SumElapsed += avg * float64(sub.Total)
			item.Query += sub.Query
			item.Reuse += sub.Reuse
			item.PlatformError += sub.PlatformError
			item.ProviderError += sub.ProviderError
			item.Charge += sub.Charge
			item.Valid += sub.Valid
			item.ApiName = sub.ApiName
			item.NonredundantQuery += sub.NonredundantQuery
			item.Nonredundant += sub.NonredundantQuery
			item.ChargeReuse += sub.ChargeReuse
		}
		if item.Valid != 0 {
			item.QueryRate = strconv.FormatFloat(float64(100*item.Query)/float64(item.Valid), 'f', 2, 64) + "%"
			item.SuccessRate = strconv.FormatFloat(float64(100*item.Success)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ReuseRate = strconv.FormatFloat(float64(100*item.Reuse)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ChargeReuseRate = strconv.FormatFloat(float64(100*item.ChargeReuse)/float64(item.Valid), 'f', 2, 64) + "%"
			item.PlatformErrorRate = strconv.FormatFloat(float64(100*item.PlatformError)/float64(item.Valid), 'f', 2, 64) + "%"
			item.ProviderErrorRate = strconv.FormatFloat(float64(100*item.ProviderError)/float64(item.Valid), 'f', 2, 64) + "%"

		} else {
			item.ChargeReuseRate = "0%"
			item.QueryRate = "0%"
			item.SuccessRate = "0%"
			item.ReuseRate = "0%"
		}
		if item.Nonredundant != 0 {
			item.NonredundantQueryRate = strconv.FormatFloat(float64(100*item.NonredundantQuery)/float64(item.Nonredundant), 'f', 2, 64) + "%"
		} else {
			item.NonredundantQueryRate = "0%"
		}
		if item.Total != 0 {
			item.AvgElapsed = item.SumElapsed / float64(item.Total)
			item.ChargeRate = strconv.FormatFloat(float64(100*item.Charge)/float64(item.Total), 'f', 2, 64) + "%"
			item.ValidRate = strconv.FormatFloat(float64(100*item.Valid)/float64(item.Total), 'f', 2, 64) + "%"
		} else {
			item.ValidRate = "0%"
			item.ChargeRate = "0%"
			item.AvgElapsed = 0
		}
		ret.LogQueryUserAcessCount = append(ret.LogQueryUserAcessCount, item)

	}
	return ret, nil
}

func getUserAccessCountExportFromReport(startDate, endDate string) (*apis.LogQueryUserAccessCountExportReply, error) {
	tabs := []TGdReport{}
	ret := &apis.LogQueryUserAccessCountExportReply{}
	o := orm.NewOrm()
	sql := fmt.Sprintf("select * from t_gd_report where date >= '%s' and date <='%s' and repeated=0", startDate, endDate)
	_, err := o.Raw(sql).QueryRows(&tabs)

	if err != nil {
		l.Error("sql",
			zap.String("call", "getUserAccessCountExportFromReport"),
			zap.String("args", sql),
			zap.String("error", err.Error()))
		return nil, errors.DataBaseError
	}
	for _, v := range tabs {
		item := apis.LogQueryUserAcessCountExport{}
		item.Date = v.Date
		item.Total = v.Total
		item.MerchantId = v.MerchantId
		item.ApiId = v.ApiId
		item.Success = v.Success
		item.Charge = v.Charge
		item.Valid = v.Valid
		avgElapsed, _ := strconv.ParseFloat(v.AvgElapsed, 64)
		item.SumElapsed = avgElapsed * float64(v.Total)
		item.AvgElapsed, _ = strconv.ParseFloat(v.AvgElapsed, 64)
		item.Query = v.Query
		item.MerchantName = v.MerchantName
		item.ApiName = v.ApiName
		item.ValidRate = v.ValidRate
		item.ChargeRate = v.ChargeRate
		item.ProviderErrorRate = v.ProviderErrorRate
		item.ProviderError = v.ProviderError
		item.PlatformErrorRate = v.PlatformErrorRate
		item.PlatformError = v.PlatformError
		item.Reuse = v.Reuse
		item.ReuseRate = v.ReuseRate
		item.QueryRate = v.QueryRate
		item.SuccessRate = v.SuccessRate
		item.TimeoutCount = v.TimeoutCount
		item.TimeoutInfo = v.TimeoutInfo
		item.NonredundantQuery = v.NonredundantQuery
		item.Nonredundant = v.Nonredundant
		item.NonredundantQueryRate = v.NonredundantQueryRate
		item.ChargeReuse = v.ChargeReuse
		item.ChargeReuseRate = v.ChargeReuseRate
		item.ChargeRate = v.ChargeRate
		ret.LogQueryUserAcessCount = append(ret.LogQueryUserAcessCount, item)
	}

	return ret, nil
}

func exportWeek(weekStart string) (string, error) {
	start, end := getWeekExportTimestamp(weekStart)
	file := xlsx.NewFile()
	detailSheet, err := file.AddSheet("明细")
	if err != nil {
		l.Error("func",
			zap.String("call", "file.AddSheet"),
			zap.String("args", "明细"),
			zap.String("error", err.Error()))
		return "", err
	}
	makeXlsxDetailHeader(detailSheet, "", false)
	chargeMap := map[string]int64{}

	// 所有商户报表
	merchantMap := map[int64][]apis.LogQueryUserAcessCountExport{}
	lastMerchantMap := map[int64][]apis.LogQueryUserAcessCountExport{}
	sheet, err := file.AddSheet("总览")
	if err != nil {
		l.Error("func",
			zap.String("call", "file.AddSheet"),
			zap.String("args", "总览"),
			zap.String("error", err.Error()))
		return "", err
	}
	makeXlsxViewHeader(sheet, "", true, false)

	startDate := time.Unix(start, 0).Format("2006-01-02")
	endDate := time.Unix(end, 0).AddDate(0, 0, -1).Format("2006-01-02")
	reply, err := getUserAccessCountExportFromReport(startDate, endDate)
	if err != nil {
		return "", err
	}

	for _, v := range reply.LogQueryUserAcessCount {
		if v.NotShow {
			continue
		}
		merchantMap[v.MerchantId] = append(merchantMap[v.MerchantId], v)
	}
	for id, array := range merchantMap {
		name := array[0].MerchantName
		if strings.Contains(name, "成都小时代科技有限公司") || name == "hj-test" || name == "DB" || name == "GD" || name == "DC" {
			lastMerchantMap[id] = array
			continue
		}
		if err := parseAccessCountDetailNew(array, detailSheet, "", name, chargeMap); err != nil {
			return "", err
		}
	}
	for _, array := range lastMerchantMap {
		name := array[0].MerchantName
		if err := parseAccessCountDetailNew(array, detailSheet, "", name, chargeMap); err != nil {
			return "", err
		}
	}

	if err := parseAccessCountView(reply, sheet, "", chargeMap, ""); err != nil {
		return "", err
	}
	filename := fmt.Sprintf("gd平台周报-%s-%s",
		startDate,
		endDate)
	path := config.Conf.ReportLocalDir + filename + ".xlsx"
	setAllCells(file)
	err = file.Save(path)
	if err != nil {
		l.Error("func",
			zap.String("call", "save excel"),
			zap.String("file_name", path),
			zap.String("error", err.Error()))
	}
	return path, err
}

func exportMonth(monthStart string) (string, string, error) {
	start, end := getMonthExportTimestamp(monthStart)
	startDay := time.Unix(start, 0).Format("2006-01-02")
	endDay := time.Unix(end, 0).AddDate(0, 0, -1).Format("2006-01-02")
	// 平台月报
	file := xlsx.NewFile()

	// 商户月报文件集
	merchantFiles := []string{}
	detailsheet, err := file.AddSheet("明细")
	if err != nil {
		l.Error("func",
			zap.String("call", "file.AddSheet"),
			zap.String("args", "明细"),
			zap.String("error", err.Error()))
		return "", "", err
	}
	makeXlsxDetailHeader(detailsheet, "", true)
	// 单个商户报表

	merchantMap := map[int64][]apis.LogQueryUserAcessCountExport{}

	reply, err := getUserAccessCountExportFromReport(startDay, endDay)
	if err != nil {
		return "", "", err
	}
	for _, v := range reply.LogQueryUserAcessCount {
		if v.NotShow {
			continue
		}
		merchantMap[v.MerchantId] = append(merchantMap[v.MerchantId], v)
	}
	for _, array := range merchantMap {
		name := array[0].MerchantName
		merchantFile := xlsx.NewFile()
		if err := parseAccessCount(array, merchantFile, ""); err != nil {
			return "", "", err
		}
		merchantFileName := fmt.Sprintf("gd平台-%s-%s-月报", name, time.Unix(start, 0).Format("2006-01"))
		merchantFilePath := config.Conf.ReportLocalDir + merchantFileName + ".xlsx"
		setAllCells(merchantFile)
		merchantFile.Save(merchantFilePath)
		merchantFiles = append(merchantFiles, merchantFilePath)

	}

	// 所有商户报表
	sheet, err := file.AddSheet("总览")
	if err != nil {
		l.Error("func",
			zap.String("call", "file.AddSheet"),
			zap.String("args", "总览"),
			zap.String("error", err.Error()))
		return "", "", err
	}
	makeXlsxViewHeader(sheet, "", true, false)

	reply, err = getUserAccessCountExportFromReportGroupByApi(startDay, endDay)
	if err != nil {
		return "", "", err
	}

	if err := parseAccessCountMonthDetail(reply, detailsheet, "", start); err != nil {
		return "", "", err
	}
	if err := parseAccessCountView(reply, sheet, "", nil, ""); err != nil {
		return "", "", err
	}
	// 平台月报只有一个文件
	filename := fmt.Sprintf("gd平台月报-%s", time.Unix(start, 0).Format("2006-01"))
	path := config.Conf.ReportLocalDir + filename + ".xlsx"
	setAllCells(file)
	err = file.Save(path)
	if err != nil {
		l.Error("func",
			zap.String("call", "save excel"),
			zap.String("file_name", path),
			zap.String("error", err.Error()))
	}

	// 商户月报需打压缩包
	zipfileName := fmt.Sprintf(config.Conf.ReportLocalDir + "gd平台商户月报-%s.zip", time.Unix(start, 0).Format("2006-01"))
	zipfile := makeZip(zipfileName, merchantFiles)
	remove(merchantFiles...)
	return path, zipfile, err
}

func removeOld() {
	prefixs := []string{
		"gd平台日报",
		"gd平台周报",
		"gd平台月报",
		"gd平台商户月报",
		"gd数据源日报",
		"gd数据源月报",
		"gd数据源周报",
		"gd商务日报",
		"gd商务周报",
		"gd_商务月报",
	}
	list, _ := ioutil.ReadDir(config.Conf.ReportLocalDir)
	for _, v := range list {
		name := v.Name()
		for _, prefix := range prefixs {
			if strings.Contains(name, prefix) {
				os.Remove(config.Conf.ReportLocalDir + name)
			}
		}
	}
}

func AutoExportTest(o orm.Ormer) {
	db = o
	/*
			dayProviderXls := ""
			weekProviderXls := ""
			monthProviderXls := ""
			monthProviderZip := ""
		providerTo := config.Conf.AutoExportProviderMail
	*/
	to := config.Conf.AutoExportMail
	mailHost := config.Conf.Warning.MailHost
	mailUser := config.Conf.Warning.MailUser
	mailPass := config.Conf.Warning.MailPassword
	//mailUser = "liuqizhong@utimes.cc"
	//mailPass = "Ll19871990"

	TestAuto = true
	start, end := getDayExportTimestamp()
	//dayxls, _ = exportDay(start, end, 0, 0, true)
	//weekxls = exportWeek()
	//monthxls, monthzip = exportMonth()
	//sendMail(to, mailHost, mailUser, mailPass, dayxls, weekxls, monthxls, monthzip)
	//dayProviderXls = exportProviderDay(start, end, 144)
	//weekProviderXls = exportProviderWeek()
	//monthProviderXls, monthProviderZip = exportProviderMonth()
	//sendMail(providerTo, mailHost, mailUser, mailPass, dayProviderXls, weekProviderXls, monthProviderXls, monthProviderZip)
	//remove(dayxls, weekxls, monthzip, monthxls)
	businessDay, err := exportBusinessDay(start, end, 0, 0, true)
	if err != nil {
		fmt.Printf("day err:%v\n", err)
		return
	}
	businessWeek, err := exportBusinessWeek("")
	if err != nil {
		fmt.Printf("week err:%v\n", err)
		return
	}
	businessMonth, err := exportBusinessMonth("")
	if err != nil {
		fmt.Printf("month err:%v\n", err)
		return
	}
	sendMail(to, mailHost, mailUser, mailPass, businessDay, businessWeek, businessMonth)
	TestAuto = false
	fmt.Printf("send over\n")
}

// AutoExport 自动统计日报并发送邮件
func AutoExport() {
	go func() {
		defer func () {
			if r := recover(); r != nil {
				fmt.Printf("auto export system error:%v\n", r)
				l.Error("auto export system error",
					zap.String("error", fmt.Sprintf("%v", r)))
				os.Exit(-1)
			}
		}()
		// 参数初始化
		db = orm.NewOrm()
		dayxls := ""
		weekxls := ""
		monthzip := ""
		monthxls := ""
		dayBusinessXls := ""
		weekBusinessXls := ""
		monthBusinessXls := ""
		dayProviderXls := ""
		weekProviderXls := ""
		monthProviderXls := ""
		monthProviderZip := ""

		// 自动导出并统计报表
		for {
			var err error
			// 等到凌晨1点
			wait(1, false)

			to := config.Conf.AutoExportMail
			providerTo := config.Conf.AutoExportProviderMail
			exceptionTo := config.Conf.ExceptionMail
			businessTo := config.Conf.AutoExportBusinessMail
			mailHost := config.Conf.Warning.MailHost
			mailUser := config.Conf.Warning.MailUser
			mailPass := config.Conf.Warning.MailPassword
			paramEmpty := false

			// 邮件相关参数检查
			if to == "" ||
				mailHost == "" ||
				mailUser == "" ||
				mailPass == "" {

				l.Error("func",
					zap.String("call", "AutoExport"),
					zap.String("error", "args is empty"))
				paramEmpty = true
			}

			tmpf := xlsx.NewFile()
			tmpSheet, _ = tmpf.AddSheet("-")

			if isFirstDay() {
				removeOld()
			}
			// 商户日报
			start, end := getDayExportTimestamp()
			dayxls, err = exportDay(start, end, 0, 0, true)
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商户日报统计失败:%v\n", err.Error()))
			}

			// 商务日报
			dayBusinessXls, err = exportBusinessDay(start, end, 0, 0, true)
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商务日报统计失败:%v\n", err.Error()))
			}

			// 数据源日报
			dayProviderXls, err = exportProviderDay(start, end, 0, true)
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("数据源日报统计失败:%v\n", err.Error()))
			}
			tmpFile = config.Conf.ReportLocalDir + "tmp.xlsx"
			tmpf.Save(tmpFile)
			tmpSheet = nil
			if paramEmpty == true {
				wait(8, true)
				continue
			}

			// 商户周报
			weekxls = ""
			if isMonday() {
				weekxls, err = exportWeek("")
				if err != nil {
					sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商户周报统计失败:%v\n", err.Error()))
				}
			}

			// 商户月报
			monthxls = ""
			monthzip = ""
			if isFirstDay() {
				monthxls, monthzip, err = exportMonth("")
				if err != nil {
					sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商户月报统计失败:%v\n", err.Error()))
				}
			}

			// 商务周报
			weekBusinessXls = ""
			if isMonday() {
				weekBusinessXls, err = exportBusinessWeek("")
				if err != nil {
					sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商务周报统计失败:%v\n", err.Error()))
				}
			}

			// 商务月报
			monthBusinessXls = ""
			if isFirstDay() {
				monthBusinessXls, err = exportBusinessMonth("")
				if err != nil {
					sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商务月报统计失败:%v\n", err.Error()))
				}
			}

			// 数据源周报
			weekProviderXls = ""
			if isMonday() {
				weekProviderXls, err = exportProviderWeek("")
				if err != nil {
					sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("数据源周报统计失败:%v\n", err.Error()))
				}
			}

			// 数据源月报
			monthProviderXls = ""
			monthProviderZip = ""
			if isFirstDay() {
				monthProviderXls, monthProviderZip, err = exportProviderMonth("")
				if err != nil {
					sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("数据源月报统计失败:%v\n", err.Error()))
				}

			}
			// 等到早上8点
			wait(8, true)
			if err := sendMail(to, mailHost, mailUser, mailPass, dayxls, weekxls, monthxls, monthzip); err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商户报表发送失败:%v\n", err.Error()))
			}
			if err := sendProviderMail(providerTo, mailHost, mailUser, mailPass, dayProviderXls, weekProviderXls, monthProviderXls, monthProviderZip); err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("数据源报表发送失败:%v\n", err.Error()))
			}
			if err := sendMail(businessTo, mailHost, mailUser, mailPass, dayBusinessXls, weekBusinessXls, monthBusinessXls); err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, fmt.Sprintf("商务报表发送失败:%v\n", err.Error()))
			}
			//remove(dayxls, weekxls, monthzip, monthxls)
		}

	}()
}

// TimestampStruct 时间戳结构体
type TimestampStruct struct {
	StartTimestamp int64
	EndTimestamp   int64
	Date           string
}

// getReportDates 将日期数组转化为时间戳结构体数组， array[0]表示开始日期，array[1]表示结束日期
func getReportDates(array []string) []TimestampStruct {
	layout := "2006-01-02"
	loc, _ := time.LoadLocation("Local")
	ret := []TimestampStruct{}
	startDate := array[0]
	endDate := array[0]
	if len(array) == 2 {
		endDate = array[1]
	}

	tstart, _ := time.ParseInLocation(layout, startDate, loc)
	tend, _ := time.ParseInLocation(layout, endDate, loc)
	startUnix := tstart.Unix()
	endUnix := tend.Unix()

	for dayStart := startUnix; dayStart <= endUnix; dayStart += 24 * 3600 {
		dayEnd := dayStart + 24*3600
		item := TimestampStruct{
			Date:           time.Unix(dayStart, 0).Format(layout),
			StartTimestamp: dayStart,
			EndTimestamp:   dayEnd,
		}
		ret = append(ret, item)
	}
	return ret
}

func pathExists(path string) bool {
	_, err := os.Stat(path)
	if err == nil {
		return true
	}
	return false
}

// LogGenerateReportOnlyMail 手动发送日报邮件接口
func LogGenerateReportOnlyMail(req *apis.LogGenerateReportReq) error {
	to := config.Conf.AutoExportMail
	providerTo := config.Conf.AutoExportProviderMail
	businessTo := config.Conf.AutoExportBusinessMail
	mailHost := config.Conf.Warning.MailHost
	mailUser := config.Conf.Warning.MailUser
	mailPass := config.Conf.Warning.MailPassword
	if req.FileName != "" {
		path := config.Conf.ReportLocalDir + req.FileName
		if pathExists(path) == false {
			return jsonrpc2.NewJsonError(10002, fmt.Sprintf("%s 文件不存在", path))
		}
		if err := sendMail(to, mailHost, mailUser, mailPass, path, "", "", ""); err != nil {
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
		return nil
	}

	if req.ApiReport {
		filename := fmt.Sprintf("gd平台日报-%s", time.Now().AddDate(0, 0, -1).Format("2006-01-02"))
		path := config.Conf.ReportLocalDir + filename + ".xlsx"
		if pathExists(path) == false {
			return jsonrpc2.NewJsonError(10002, fmt.Sprintf("%s 文件不存在", path))
		}
		if err := sendMail(to, mailHost, mailUser, mailPass, path, "", "", ""); err != nil {
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}
	if req.ProviderReport {
		filename := fmt.Sprintf("gd数据源日报-%s", time.Now().AddDate(0, 0, -1).Format("2006-01-02"))
		path := config.Conf.ReportLocalDir + filename + ".xlsx"
		if pathExists(path) == false {
			return jsonrpc2.NewJsonError(10002, fmt.Sprintf("%s 文件不存在", path))
		}
		if err := sendProviderMail(providerTo, mailHost, mailUser, mailPass, path, "", "", ""); err != nil {
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}

	if req.BusinessReport {
		filename := fmt.Sprintf("gd商务日报-%s", time.Now().AddDate(0, 0, -1).Format("2006-01-02"))
		path := config.Conf.ReportLocalDir + filename + ".xlsx"
		if pathExists(path) == false {
			return jsonrpc2.NewJsonError(10002, fmt.Sprintf("%s 文件不存在", path))
		}
		if err := sendMail(businessTo, mailHost, mailUser, mailPass, path, "", "", ""); err != nil {
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}

	return nil
}

// LogGenerateReport 手动统计日报和发送邮件接口
func LogGenerateReport(ctx context.Context, req *apis.LogGenerateReportReq, reply *apis.LogGenerateReportReply) error {
	to := config.Conf.AutoExportMail
	providerTo := config.Conf.AutoExportProviderMail
	mailHost := config.Conf.Warning.MailHost
	mailUser := config.Conf.Warning.MailUser
	mailPass := config.Conf.Warning.MailPassword
	exceptionTo := config.Conf.ExceptionMail
	businessTo := config.Conf.AutoExportBusinessMail

	if req.OnlyMail {
		return LogGenerateReportOnlyMail(req)
	}

	array := strings.Split(req.Dates, ";")
	if req.Dates == "" {
		array = []string{time.Now().AddDate(0, 0, -1).Format("2006-01-02")}
	}
	if len(array) == 0 || len(array) > 2 {
		return errors.ArgsError
	}
	times := getReportDates(array)
	if req.MysqlAddr != "" {
		dataSource := fmt.Sprintf("%s:%s@tcp(%s)/%s?charset=%s", req.MysqlUser, req.MysqlPass, req.MysqlAddr, "db_gd_access_log", "utf8")
		if err := orm.RegisterDataBase("local", "mysql", dataSource, 2, 2); err != nil {
			return errors.DataBaseError
		}
	}
	db = orm.NewOrm()
	if req.MysqlAddr != "" {
		db.Using("local")
	}
	for _, v := range times {
		if req.ApiReport {
			apiReportFile, err := exportDay(v.StartTimestamp, v.EndTimestamp, req.MerchantId, req.ApiId, req.IsInsert)
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
				return jsonrpc2.NewJsonError(10002, "商户日报统计失败:"+err.Error())
			}
			if req.OnlyReport {
				return nil
			}
			err = sendMail(to, mailHost, mailUser, mailPass, apiReportFile, "", "", "")
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
				return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
			}
		}
		if req.ProviderReport {
			providerReportFile, err := exportProviderDay(v.StartTimestamp, v.EndTimestamp, req.ProviderApiId, req.IsInsert)
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
				return jsonrpc2.NewJsonError(10002, "数据源日报统计失败:"+err.Error())
			}
			if req.OnlyReport {
				return nil
			}
			err = sendProviderMail(providerTo, mailHost, mailUser, mailPass, providerReportFile, "", "", "")
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
				return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
			}
		}

		if req.BusinessReport {
			apiReportFile, err := exportBusinessDay(v.StartTimestamp, v.EndTimestamp, req.MerchantId, req.ApiId, req.IsInsert)
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
				return jsonrpc2.NewJsonError(10002, "商务日报统计失败:"+err.Error())
			}
			if req.OnlyReport {
				return nil
			}
			err = sendMail(businessTo, mailHost, mailUser, mailPass, apiReportFile, "", "", "")
			if err != nil {
				sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
				return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
			}
		}
	}


	if req.ApiWeekStart != "" {
		week, err := exportWeek(req.ApiWeekStart)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "api周报报统计失败:"+err.Error())
		}
		err = sendMail(to, mailHost, mailUser, mailPass, week, "", "", "")
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}


	if req.ProviderWeekStart != "" {
		week, err := exportProviderWeek(req.ProviderWeekStart)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "provider周报报统计失败:"+err.Error())
		}
		err = sendProviderMail(providerTo, mailHost, mailUser, mailPass, week, "", "", "")
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}

	if req.BusinessWeekStart != "" {
		week, err := exportBusinessWeek(req.BusinessWeekStart)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "商务周报报统计失败:"+err.Error())
		}
		err = sendMail(businessTo, mailHost, mailUser, mailPass, week, "", "", "")
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}

	if req.ApiMonthStart != "" {
		month, monthzip, err := exportMonth(req.ApiMonthStart)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "api月报报统计失败:"+err.Error())
		}
		err = sendMail(to, mailHost, mailUser, mailPass, month, monthzip)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}

	if req.ProviderMonthStart != "" {
		month, monthzip, err := exportProviderMonth(req.ProviderMonthStart)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "provider月报报统计失败:"+err.Error())
		}
		err = sendProviderMail(providerTo, mailHost, mailUser, mailPass, month, monthzip)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}

	if req.BusinessMonthStart != "" {
		month, err := exportBusinessMonth(req.BusinessMonthStart)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "商务月报报统计失败:"+err.Error())
		}
		err = sendMail(businessTo, mailHost, mailUser, mailPass, month)
		if err != nil {
			sendExceptionMail(exceptionTo, mailHost, mailUser, mailPass, err.Error())
			return jsonrpc2.NewJsonError(10002, "邮件发送失败:"+err.Error())
		}
	}


	return nil
}

// LocalGenerateReport 用于日报统计(本地执行)
func LocalGenerateReport(req *apis.LogGenerateReportReq) {
	bytes, _ := json.Marshal(*req)
	fmt.Printf("local generate report:%s\n", bytes)
	array := strings.Split(req.Dates, ";")
	if len(array) == 0 || len(array) > 2 {
		fmt.Printf("参数错误\n")
		return
	}
	times := getReportDates(array)
	if req.MysqlAddr != "" {
		dataSource := fmt.Sprintf("%s:%s@tcp(%s)/%s?charset=%s", req.MysqlUser, req.MysqlPass, req.MysqlAddr, "db_gd_access_log", "utf8")
		if err := orm.RegisterDataBase("local", "mysql", dataSource, 2, 2); err != nil {
			fmt.Printf("%v\n", err.Error())
			return
		}
	}
	db = orm.NewOrm()
	if req.MysqlAddr != "" {
		db.Using("local")
	}
	for _, v := range times {
		fmt.Printf("date:%v\n", v)
		if req.ApiReport {
			exportDay(v.StartTimestamp, v.EndTimestamp, req.MerchantId, req.ApiId, req.IsInsert)
		}
		if req.ProviderReport {
			exportProviderDay(v.StartTimestamp, v.EndTimestamp, req.ProviderApiId, req.IsInsert)
		}
	}
	fmt.Printf("local generate report over\n")
}
