// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

/*
 pacakage warning 提供用量预警和日志预警功能
*/
package warning

import (
	"context"
	"gd_crontab/apis"
	"fmt"
	"strconv"
	"strings"
	"sync"
	"time"

	"gd_crontab/common.in/config"
	"gd_crontab/common.in/mongo"
	"gopkg.in/mgo.v2"
	"gopkg.in/mgo.v2/bson"
)

// apiCountTab 记录api用量预警信息(按百分比预警)的集合
var apiCountTab = "t_gd_access_count"

// apiCountTab 记录api用量预警信息(按具体数值)的集合
var apiCountTabSpecial = "t_gd_access_count_special"

// providerGlobalTab 记录数据源用量预警信息的集合
var providerGlobalTab = "t_gd_thirdpart_global_count"

type MgoApiCount struct {
	Id                     bson.ObjectId `json:"_id" bson:"_id"`
	MerchantDataApiId      int64         `json:"merchant_data_api_id" bson:"merchant_data_api_id"`
	RemainRateThreshold    float64       `json:"remain_rate_threshold" bson:"remain_rate_threshold"`
	DayRemainRateThreshold float64       `json:"day_remain_rate_threshold" bson:"day_remain_rate_threshold"`
	RemainDaysThreshold    int           `json:"remain_days_threshold" bson:"remain_days_threshold"`
}

type MgoApiCountSpecial struct {
	Id                  bson.ObjectId `json:"_id" bson:"_id"`
	MerchantDataApiId   int64         `json:"merchant_data_api_id" bson:"merchant_data_api_id"`
	RemainThreshold     int           `json:"remain_threshold" bson:"remain_threshold"`
	DayRemainThreshold  int           `json:"day_remain_threshold" bson:"day_remain_threshold"`
	RemainDaysThreshold int           `json:"remain_days_threshold" bson:"remain_days_threshold"`
}

var (
	ApiTotalCountThresholds    = []float64{}
	ApiDayCountThresholds      = []float64{}
	ApiDayThresholds           = []int{}
	ProviderDayCountThresholds = []float64{}

	ApiLogAvgElapsed        = float64(5.0)
	ApiLogFailThreshold     = float64(0.2)
	ApiLogNorecordThreshold = float64(0.2)

	ProviderLogAvgElapsed        = float64(5.0)
	ProviderLogFailThreshold     = float64(0.2)
	ProviderLogNorecordThreshold = float64(0.2)

	LogTimeInterval = 15
)

// parseThresholdsFromReq从请求参数中解析阀值
func parseThresholdsFromReq(thresholds string) (ret []int) {
	array := strings.Split(thresholds, ",")
	ret = make([]int, len(array))
	for i, v := range array {
		value, _ := strconv.Atoi(v)
		ret[i] = value
	}
	for i := 0; i < len(ret); i++ {
		for j := i + 1; j < len(ret); j++ {
			if ret[i] < ret[j] {
				tmp := ret[i]
				ret[i] = ret[j]
				ret[j] = tmp
			}
		}
	}
	return ret
}

// ParseThreshold从etcd配置中解析阀值信息
func ParseThreshold() {
	ApiTotalCountThresholds = stringToFloatArray(config.Conf.Warning.ApiTotalCountThresholds)
	ApiDayCountThresholds = stringToFloatArray(config.Conf.Warning.ApiDayCountThresholds)
	ApiDayThresholds = stringToIntArray(config.Conf.Warning.ApiDayThresholds)
	ProviderDayCountThresholds = stringToFloatArray(config.Conf.Warning.ProviderDayCountThresholds)

	f, _ := strconv.ParseFloat(config.Conf.Warning.ApiLogAvgElapsed, 64)
	if f > 0 {
		ApiLogAvgElapsed = f
	}

	f, _ = strconv.ParseFloat(config.Conf.Warning.ApiLogFailThreshold, 64)
	if f > 0 {
		ApiLogFailThreshold = f
	}

	f, _ = strconv.ParseFloat(config.Conf.Warning.ApiLogNorecordThreshold, 64)
	if f > 0 {
		ApiLogNorecordThreshold = f
	}

	f, _ = strconv.ParseFloat(config.Conf.Warning.ProviderLogAvgElapsed, 64)
	if f > 0 {
		ProviderLogAvgElapsed = f
	}

	f, _ = strconv.ParseFloat(config.Conf.Warning.ProviderLogFailThreshold, 64)
	if f > 0 {
		ProviderLogFailThreshold = f
	}

	f, _ = strconv.ParseFloat(config.Conf.Warning.ProviderLogNorecordThreshold, 64)
	if f > 0 {
		ProviderLogNorecordThreshold = f
	}
	i, _ := strconv.Atoi(config.Conf.Warning.LogTimeInterval)
	if i > 0 {
		LogTimeInterval = i
	}
}

// stringToFloatArray 将data转为float数组, 1.1；2.2 转为 [1.1, 2.2]
func stringToFloatArray(data string) []float64 {
	array := strings.Split(data, ";")
	ret := make([]float64, len(array))
	for i, v := range array {
		value, _ := strconv.ParseFloat(v, 64)
		ret[i] = value
	}
	return ret
}

func stringToIntArray(data string) []int {
	array := strings.Split(data, ";")
	ret := make([]int, len(array))
	for i, v := range array {
		value, _ := strconv.Atoi(v)
		ret[i] = value
	}
	return ret
}

// apiCountMutex 互斥访问apiNameMap的锁
var apiCountMutex sync.Mutex

// providerCountMutex 互斥访问providerNameMap的锁
var providerCountMutex sync.Mutex
var apiNameMap = map[string]string{}
var providerNameMap = map[string]string{}

// lock ltype 0：api， 1：数据源信息， name代表锁标识
func lock(name string, ltype int) {
	if ltype == 0 {
		for {
			apiCountMutex.Lock()
			if _, ok := apiNameMap[name]; ok == false {
				apiNameMap[name] = "-"
				apiCountMutex.Unlock()
				return

			}
			apiCountMutex.Unlock()
			time.Sleep(50 * time.Millisecond)
		}
	}

	for {
		providerCountMutex.Lock()
		if _, ok := providerNameMap[name]; ok == false {
			providerNameMap[name] = "-"
			providerCountMutex.Unlock()
			return

		}
		providerCountMutex.Unlock()
		time.Sleep(50 * time.Millisecond)
	}

}

// unLock ltype 0：api， 1：数据源信息， name代表锁标识
func unLock(name string, ltype int) {
	if ltype == 0 {
		apiCountMutex.Lock()
		if _, ok := apiNameMap[name]; ok == true {
			delete(apiNameMap, name)
		}
		apiCountMutex.Unlock()
		return

	}

	providerCountMutex.Lock()
	if _, ok := providerNameMap[name]; ok == true {
		delete(providerNameMap, name)
	}
	providerCountMutex.Unlock()
}

// getMinFloatThresh 从array中获取比remain大的最小值
func getMinFloatThresh(array []float64, remain float64) float64 {
	min := 0.0
	for _, v := range array {
		if remain >= v {
			continue
		}
		if min == 0 {
			min = v
		}
		if min > v {
			min = v
		}
	}
	return min
}

// getMinIntThresh 从array中获取比remain大的最小值
func getMinIntThresh(array []int, remain int) int {
	min := 0
	for _, v := range array {
		if remain >= v {
			continue
		}
		if min == 0 {
			min = v
		}
		if min > v {
			min = v
		}
	}
	return min
}

// WarningApiCountByDay 按天的api用量(按百分比)预警，仅发送给etcd配置的收件人， checkDays 表示是否对天数过期进行预警
func WarningApiCountByDay(req *apis.WarningApiCountReq, collection *mgo.Collection, checkDays bool) {
	text := ""
	w := apis.Warning{}
	now := time.Now().Unix()

	// etcd 默认阀值
	var remainDayConf = ApiDayThresholds
	var remainDayCountConf = ApiDayCountThresholds

	// 计算今日剩余量 和剩余天数
	remainDayCount := req.DayCount - req.DayUsed
	if remainDayCount < 0 {
		remainDayCount = 0
	}
	remainDay := int((req.EndTime-now)/(3600*24)) + 1
	remainDayRate := float64(remainDayCount) / float64(req.DayCount)
	w.To = strings.Split(config.Conf.Warning.DefaultMails, ";")
	if remainDay < 0 {
		remainDay = 0
	}

	// 未达阈值直接返回
	if remainDay >= remainDayConf[0] && remainDayRate >= remainDayCountConf[0] {
		collection.Remove(bson.M{"merchant_data_api_id": req.MerchantDataApiId})
		return
	}

	// 对key加锁， 避免并发引起的重复邮件发送
	key := fmt.Sprintf("%d", req.MerchantDataApiId)
	lock(key, 0)
	defer unLock(key, 0)
	mac := MgoApiCount{}
	collection.Find(bson.M{"merchant_data_api_id": req.MerchantDataApiId}).One(&mac)
	defer func() {
		// 预警信息记入mgo, 并发送邮件
		if text == "" {
			return
		}
		if mac.Id.Valid() == false {
			mac.MerchantDataApiId = req.MerchantDataApiId
			mongo.MgoInsertMulti(collection, true, mac)
		} else {
			collection.UpdateId(mac.Id, mac)
		}
		if checkDays == false && (strings.Contains(text, "过期") ||
			strings.Contains(text, "天数")) {
			return
		}
		w.WarningText = append(w.WarningText,text)
		//w.WarningText = text
		Warning(&w)
	}()

	// 检查剩余天数，若对同一阈值已告警则不再重复告警
	if remainDay == 0 && mac.Id.Valid() && mac.RemainDaysThreshold == 0 {
		return
	}
	if remainDay == 0 && (mac.Id.Valid() == false || mac.RemainDaysThreshold > 0) {
		text = fmt.Sprintf("平台用户%s的%s接口已过期，请关注", req.User, req.ApiName)
		mac.RemainDaysThreshold = 0
		w.Subject = fmt.Sprintf("%s %s 平台api用量告警", req.User, req.ApiName)
		return
	}
	minDayThresh := getMinIntThresh(remainDayConf, remainDay)
	if remainDay < minDayThresh && (mac.RemainDaysThreshold != minDayThresh || mac.RemainDaysThreshold == 0) {
		text = fmt.Sprintf("平台用户%s的%s接口剩余有效天数仅有%d天，请关注", req.User, req.ApiName, remainDay)
		mac.RemainDaysThreshold = minDayThresh
		w.Subject = fmt.Sprintf("%s %s 平台api用量告警", req.User, req.ApiName)
		return
	}

	// 检查今日剩余次数，若对同一阈值已告警则不再重复告警
	if remainDayRate == 0 && mac.Id.Valid() && mac.DayRemainRateThreshold == 0 {
		return
	}
	if remainDayRate == 0 && (mac.Id.Valid() == false || mac.DayRemainRateThreshold > 0) {
		text = fmt.Sprintf("平台用户%s的%s接口今日用量已达上限，请关注", req.User, req.ApiName)
		mac.DayRemainRateThreshold = 0
		w.Subject = fmt.Sprintf("%s %s 平台api用量告警", req.User, req.ApiName)
		return
	}
	minCountThreshold := getMinFloatThresh(remainDayCountConf, remainDayRate)

	if remainDayRate < minCountThreshold && (mac.DayRemainRateThreshold != minCountThreshold || mac.DayRemainRateThreshold == 0) {
		text = fmt.Sprintf("平台用户%s的%s接口今日余量已不足%d%%, 剩余用量%d，请关注",
			req.User, req.ApiName, int(100.0*(minCountThreshold)), remainDayCount)
		mac.DayRemainRateThreshold = minCountThreshold
		w.Subject = fmt.Sprintf("%s %s 平台api用量告警", req.User, req.ApiName)
		return
	}
}

// WarningApiSpecialCountByDay 按天的api用量(按具体数值)预警，如果商户配置了邮箱则发送给商户,
// 否则发送给etcd配置的收件人
func warningApiSpecialCountByDay(req *apis.WarningApiCountReq, collection *mgo.Collection) {
	text := ""
	w := apis.Warning{}
	now := time.Now().Unix()
	var remainDayConf = parseThresholdsFromReq(req.DayThresholds)
	var remainDayCountConf = parseThresholdsFromReq(req.CountThresholds)

	// 计算今日剩余量 和剩余天数
	remainDayCount := req.DayCount - req.DayUsed
	if remainDayCount < 0 {
		remainDayCount = 0
	}
	remainDay := int((req.EndTime-now)/(3600*24)) + 1

	if req.Email == "" {
		w.To = strings.Split(config.Conf.Warning.DefaultMails, ";")
	} else {
		w.To = append(w.To, req.Email)
	}
	if remainDay < 0 {
		remainDay = 0
	}

	// 未达阈值
	if remainDay >= remainDayConf[0] && remainDayCount >= remainDayCountConf[0] {
		collection.Remove(bson.M{"merchant_data_api_id": req.MerchantDataApiId})
		return
	}
	key := fmt.Sprintf("%d-special", req.MerchantDataApiId)
	lock(key, 0)
	defer unLock(key, 0)
	mac := MgoApiCountSpecial{}
	collection.Find(bson.M{"merchant_data_api_id": req.MerchantDataApiId}).One(&mac)
	defer func() {
		if text == "" {
			return
		}
		if mac.Id.Valid() == false {
			mac.MerchantDataApiId = req.MerchantDataApiId
			mongo.MgoInsertMulti(collection, true, mac)
		} else {
			collection.UpdateId(mac.Id, mac)
		}
		w.WarningText = append(w.WarningText,text)
		//w.WarningText = text
		Warning(&w)
	}()

	// 检查剩余天数，若对同一阈值已告警则不再重复告警
	if remainDay == 0 && mac.Id.Valid() && mac.RemainDaysThreshold == 0 {
		return
	}
	if remainDay == 0 && (mac.Id.Valid() == false || mac.RemainDaysThreshold > 0) {
		text = fmt.Sprintf("平台用户%s的%s接口已过期，请关注", req.User, req.ApiName)
		mac.RemainDaysThreshold = 0
		w.Subject = fmt.Sprintf("%s %s 平台api用量提示", req.User, req.ApiName)
		return
	}
	minDayThresh := getMinIntThresh(remainDayConf, remainDay)
	if remainDay < minDayThresh && (mac.RemainDaysThreshold != minDayThresh || mac.RemainDaysThreshold == 0) {
		text = fmt.Sprintf("平台用户%s的%s接口剩余有效天数仅有%d天，请关注", req.User, req.ApiName, remainDay)
		mac.RemainDaysThreshold = minDayThresh
		w.Subject = fmt.Sprintf("%s %s 平台api用量提示", req.User, req.ApiName)
		return
	}

	// 检查今日剩余次数，若对同一阈值已告警则不再重复告警
	if remainDayCount == 0 && mac.Id.Valid() && mac.DayRemainThreshold == 0 {
		return
	}
	if remainDayCount == 0 && (mac.Id.Valid() == false || mac.DayRemainThreshold > 0) {
		text = fmt.Sprintf("平台用户%s的%s接口今日用量已达上限，请关注", req.User, req.ApiName)
		mac.DayRemainThreshold = 0
		w.Subject = fmt.Sprintf("%s %s 平台api用量提示", req.User, req.ApiName)
		return
	}
	minCountThreshold := getMinIntThresh(remainDayCountConf, remainDayCount)

	if remainDayCount < minCountThreshold && (mac.DayRemainThreshold != minCountThreshold || mac.DayRemainThreshold == 0) {
		text = fmt.Sprintf("平台用户%s的%s接口今日余量已不足%d, 剩余用量%d，请关注",
			req.User, req.ApiName, minCountThreshold, remainDayCount)
		mac.DayRemainThreshold = minCountThreshold
		w.Subject = fmt.Sprintf("%s %s 平台api用量提示", req.User, req.ApiName)
		return
	}
}

// WarningApiSpecialCountByTotal 按次的api用量(按具体数值)预警，若商户配置了邮箱则发送给商户，
// 否则发送给etcd配置的收件人
func warningApiSpecialCountByTotal(req *apis.WarningApiCountReq, collection *mgo.Collection) {
	remain := req.RealTotal - req.RealUsed

	// 获取阀值信息
	var remainCountConf = parseThresholdsFromReq(req.CountThresholds)
	if len(remainCountConf) == 0 {
		return
	}

	// 设置收件箱
	w := apis.Warning{}
	if req.Email == "" {
		w.To = strings.Split(config.Conf.Warning.DefaultMails, ";")
	} else {
		w.To = append(w.To, req.Email)
	}
	if remain < 0 {
		remain = 0
	}

	// 余量未达阀值直接返回
	if remain >= remainCountConf[0] {
		collection.Remove(bson.M{"merchant_data_api_id": req.MerchantDataApiId})
		return
	}

	key := fmt.Sprintf("%d-special", req.MerchantDataApiId)
	lock(key, 0)
	defer unLock(key, 0)
	mac := MgoApiCountSpecial{}
	collection.Find(bson.M{"merchant_data_api_id": req.MerchantDataApiId}).One(&mac)
	text := ""
	defer func() {
		// 预警信息写入mgo，并发送邮件
		if text == "" {
			return
		}
		if mac.Id.Valid() == false {
			mac.MerchantDataApiId = req.MerchantDataApiId
			mongo.MgoInsertMulti(collection, true, mac)
		} else {
			collection.UpdateId(mac.Id, mac)
		}
		w.WarningText = append(w.WarningText,text)
		//w.WarningText = text
		Warning(&w)
	}()

	// 用量耗尽，但已发送过邮件
	if remain == 0 && mac.Id.Valid() && mac.RemainThreshold == 0 {
		return
	}

	// 用量耗尽，且未发送过耗尽邮件
	if remain == 0 && (mac.Id.Valid() == false || mac.RemainThreshold > 0) {
		text = fmt.Sprintf("平台用户%s的%s接口用量已耗尽",
			req.User, req.ApiName)
		mac.RemainThreshold = 0
		w.Subject = fmt.Sprintf("%s %s 平台api用量提示", req.User, req.ApiName)
		return
	}

	// 达到莫个预警阀值且未发送过相关邮件,则准备邮件信息
	minThreshold := getMinIntThresh(remainCountConf, remain)
	if remain < minThreshold && (mac.RemainThreshold != minThreshold || mac.RemainThreshold == 0) {
		text = fmt.Sprintf("平台用户 %s 的 %s 接口用量已不足%d, 剩余用量为%d",
			req.User, req.ApiName, minThreshold, remain)
		mac.RemainThreshold = minThreshold
		w.Subject = fmt.Sprintf("%s %s 平台api用量提示", req.User, req.ApiName)
		return
	}

}

// WarningApiCountByTotal 按次的api用量(按百分比)预警，仅发送给etcd配置的收件人
func WarningApiCountByTotal(req *apis.WarningApiCountReq, collection *mgo.Collection) {
	// 获取阀值信息
	remain := req.TotalCount - req.TotalUsed
	var remainCountConf = ApiTotalCountThresholds
	if len(remainCountConf) == 0 {
		return
	}

	// 设置收件箱
	w := apis.Warning{}
	w.To = strings.Split(config.Conf.Warning.DefaultMails, ";")
	if remain < 0 {
		remain = 0
	}
	remainRate := float64(remain) / float64(req.TotalCount)
	if remainRate >= remainCountConf[0] {
		collection.Remove(bson.M{"merchant_data_api_id": req.MerchantDataApiId})
		return
	}

	key := fmt.Sprintf("%d", req.MerchantDataApiId)
	lock(key, 0)
	defer unLock(key, 0)
	mac := MgoApiCount{}
	collection.Find(bson.M{"merchant_data_api_id": req.MerchantDataApiId}).One(&mac)
	text := ""
	defer func() {
		if text == "" {
			return
		}
		if mac.Id.Valid() == false {
			mac.MerchantDataApiId = req.MerchantDataApiId
			mongo.MgoInsertMulti(collection, true, mac)
		} else {
			collection.UpdateId(mac.Id, mac)
		}
		w.WarningText = append(w.WarningText,text)
		//w.WarningText = text
		Warning(&w)
	}()

	// 用量耗尽，但已发送过邮件
	if remain == 0 && mac.Id.Valid() && mac.RemainRateThreshold == 0 {
		return
	}

	// 用量耗尽，且未发送过耗尽邮件
	if remain == 0 && (mac.Id.Valid() == false || mac.RemainRateThreshold > 0) {
		text = fmt.Sprintf("平台用户%s的%s接口用量已耗尽",
			req.User, req.ApiName)
		mac.RemainRateThreshold = 0
		w.Subject = fmt.Sprintf("%s %s 平台api用量告警", req.User, req.ApiName)
		return
	}

	// 达到莫个预警阀值且未发送过相关邮件,则准备邮件信息
	minThreshold := getMinFloatThresh(remainCountConf, remainRate)
	if remainRate < minThreshold && (mac.RemainRateThreshold != minThreshold || mac.RemainRateThreshold == 0) {
		text = fmt.Sprintf("平台用户 %s 的 %s 接口用量已不足%d%%, 剩余用量为%d",
			req.User, req.ApiName, int(100.0*(minThreshold)), remain)
		mac.RemainRateThreshold = minThreshold
		w.Subject = fmt.Sprintf("%s %s 平台api用量告警", req.User, req.ApiName)
		return
	}
}

// WarningApiCount 平台api用量预警
func WarningApiCount(ctx context.Context, req *apis.WarningApiCountReq, reply *apis.WarningApiCountReply) error {
	/*collection := mongo.Session.DB(accessDB).C(apiCountTab)
	special := mongo.Session.DB(accessDB).C(apiCountTabSpecial)
	if req.IsFree {
		collection.Remove(bson.M{"merchant_data_api_id": req.MerchantDataApiId})
		special.Remove(bson.M{"merchant_data_api_id": req.MerchantDataApiId})
		return nil
	}

	// 按次数
	if req.ComboType != 2 {
		if req.Enable == true {
			// 按百分比计算
			WarningApiCountByTotal(req, collection)
		}
		if req.CountThresholds != "" {
			// 按具体数值计算
			warningApiSpecialCountByTotal(req, special)
		}
		return nil
	}

	// 按天数
	checkDays := true
	if req.DayThresholds != "" {
		checkDays = false
	}
	if req.Enable == true {
		// 按百分比计算
		WarningApiCountByDay(req, collection, checkDays)
	}
	if req.CountThresholds != "" || req.DayThresholds != "" {
		// 按具体数值计算
		warningApiSpecialCountByDay(req, special)
	}*/
	return nil
}
